
#include "GUI.H"
#include "Game.H"
#include "SDL_gfxPrimitives.h"
#include "GameObj.H"
#include "GameTile.H"
#include "PlayerInfo.H"
#include "ServerObjData.H"
#include "Options.H"

#include "AIGUI.H"
#include "AIGlobal.H"
#include "AIEventHandler.H"
#include "AIGUI.H"
#include "Graph.H"
#include "Agent.H"
#include "Fighter.H"
#include "Path.H"
using namespace std;
using namespace bzAI;

const sint4 WATER = 1;
const sint4 GROUND = 2;
const sint4 CLIFF = 3;


static RGB colors[GameConst::PLAYER_MAX*2] = {
  RGB(0,255,0),
  RGB(255,0,0),
  RGB(0,0,255),
  RGB(255,255,0),
  RGB(0,96,0),
  RGB(96,0,0),
  RGB(0,0,96),
  RGB(96,96,0)
};

const static RGB black(0, 0, 0);
const static RGB red(255, 0, 0);
const static RGB green(0, 255, 0);
const static RGB blue(0, 0, 255);
const static RGB white(255, 255, 255);
const static RGB path_empty_color(255,0,255);	//violet

AIGUI::AIGUI(){
	GUI::GUI();
	bool op_temp;
	Options::get("-dgverts", op_temp); DISP_GVERTS = op_temp ? 1 : 0;
	Options::get("-dgedges", op_temp); DISP_GEDGES = op_temp ? 1 : 0;
        Options::get("-dtargets", op_temp); DISP_TARGETS = op_temp ? 1 : 0;
}


void AIGUI::add_options(){
	DENTER();
	GUI::add_options();
	Options o("View in 2d");
	o.put("-dgverts", "mark graph vertices");
	o.put("-dgedges", "mark graph edges");
        o.put("-dtargets", "connect units with their targets");
	DEXIT();
}

void AIGUI::display(const GameObj *obj1, const GameObj *obj2) {

	if (SDL_MUSTLOCK(screen) && SDL_LockSurface(screen) < 0 ) {
		fprintf(stderr, "Can't lock screen: %s\n", SDL_GetError());
		return;
	}

	FORALL (views, it) {
		correct_view();
		display(*it, obj1, obj2);
	}

	// draw minimap and mark active view
	//   display_minimap(views[active_view]);


	if (SDL_MUSTLOCK(screen)) SDL_UnlockSurface(screen);

	//SDL_Flip(screen);
	SDL_UpdateRect(screen, 0, 0, 0, 0);

}

Loc AIGUI::lscale(Loc l, sint4 mag){
	sint4 cx = scale(l.x, mag);
	sint4 cy = scale(l.y, mag);
	return Loc(cx, cy);
}
bool AIGUI::isVisible(Loc cl, sint4 r, const GfxView2 &gv){
	return !( (cl.x - r > gv.mx + gv.w) ||
				(cl.x + r < gv.mx) ||
				(cl.y - r > gv.my + gv.h) ||
				(cl.y + r < gv.my) 
			);
}

void AIGUI::drawGraph(const GfxView2 &gv){
	Graph *g = HANDLER->graph;

	//Draw all vertices
	FOREACH(it, g->v2coord){
		int v = it.first;
		Loc clv = lscale(it.second, gv.mag);
		sint4 r  = scale(1, gv.mag);
		if (!isVisible(clv, r, gv)) continue;		
		if (DISP_GVERTS)
			filledCircleColor(screen, clv.x - gv.mx, clv.y - gv.my, r, !g->inactive[it.first] ? black.color_alpha(): white.color_alpha());
		
		Graph::vert_vector &adjs = g->g[v];
		// Draw edges
		if (DISP_GEDGES)
		REP(i, adjs.size){
			int w = adjs.t[i];
			Loc clw = lscale(g->v2coord[w], gv.mag); 
			if (!isVisible(clw, r, gv)) continue;
			if (clv.x - clw.x > 0) 
				lineColor(screen, clv.x - gv.mx, clv.y - gv.my, clw.x - gv.mx, clw.y - gv.my, green.color_alpha());
			else if (clv.x - clw.x < 0)
				lineColor(screen, clv.x - gv.mx, clv.y - gv.my, clw.x - gv.mx, clw.y - gv.my, blue.color_alpha());
			else if (clv.y - clw.y > 0) 
				lineColor(screen, clv.x - gv.mx, clv.y - gv.my, clw.x - gv.mx, clw.y - gv.my, green.color_alpha());
			else
				lineColor(screen, clv.x - gv.mx, clv.y - gv.my, clw.x - gv.mx, clw.y - gv.my, blue.color_alpha());
		}
		
	}

	// Draw in path_empty_color Agents, which do not have a path
	FOR_EACH_OBJ_OF(aiob, Agent, player_bitmap){
		if (aiob->path->empty()){
			Loc clv = lscale(aiob->getLoc(), gv.mag);
			sint4 r = scale(aiob->gob->get_int("radius"), gv.mag);
			if (!isVisible(clv, r, gv)) continue;
			filledCircleColor(screen, clv.x - gv.mx, clv.y - gv.my, r, path_empty_color.color_alpha());
		}
                else if (DISP_TARGETS){
                        Loc clv = lscale(aiob->getLoc(), gv.mag);
                        Loc clw = lscale(aiob->getTarget(), gv.mag);
			lineColor(screen, clv.x - gv.mx, clv.y - gv.my, clw.x - gv.mx, clw.y - gv.my, white.color_alpha());
                }
	}
	FOR_EACH_OBJ_OF(aiob, Fighter, all_bitmap){
		Loc clv = lscale(aiob->getLoc(), gv.mag);
		sint4 range = scale(aiob->getRange(), gv.mag);
		sint4 r= scale(aiob->gob->get_int("radius"), gv.mag);
		if (!isVisible(clv, r, gv)) continue;
		RGB col=colors[aiob->getOwner()];
		col.from(3*aiob->gob->get_int("hp")*aiob->getOwner(),3*aiob->gob->get_int("hp")*(1-aiob->getOwner()),0);
//		col.from(3*aiob->gob->get_int("hp")*aiob->rotation_dir,3*aiob->gob->get_int("hp")*(1-aiob->rotation_dir),0);
//		col.from(3*aiob->gob->get_int("hp")*aiob->group_id,3*aiob->gob->get_int("hp")*(1-aiob->group_id),0);



		if (aiob->gob->get_int("hp")==0) cout << "!!!!!!!!!!!"<<endl;
	

		filledCircleColor(screen, clv.x - gv.mx, clv.y - gv.my, r, col.color_alpha());

	//	circleColor(screen, clv.x - gv.mx, clv.y - gv.my, r, colors[aiob->getOwner()].color_alpha());
	}
        Loc clv = lscale(Loc(928, 704), gv.mag);
        filledCircleColor(screen, clv.x - gv.mx, clv.y - gv.my, 3, black.color_alpha());
         clv = lscale(Loc(592,256), gv.mag);
        filledCircleColor(screen, clv.x - gv.mx, clv.y - gv.my, 3, black.color_alpha());
         clv = lscale(Loc(864, 719), gv.mag);
        filledCircleColor(screen, clv.x - gv.mx, clv.y - gv.my, 3, black.color_alpha());
}

void AIGUI::display(const GfxView2 &gv, const GameObj* obj1, const GameObj* obj2) {
	GUI::display(gv, obj1, obj2);
	const sint4 cid = game->get_client_player();
	//const RGB &rgb = colors[cid];
	//if (DISP_GEDGES || DISP_GVERTS)
	drawGraph(gv);
	
}

#if 0
// FIXME Do usuniecia, wklejone tylko by moc podgladac i kopiowac w obrebie jednego pliku.
void AIGUI::display2(const GfxView2 &gv, const GameObj* obj1, const GameObj* obj2) {

	sint4 tile_pixels = scale(game->get_tile_points(), gv.mag);
	SDL_Rect clip_rect;
	clip_rect.x = gv.x;
	clip_rect.y = gv.y;
	clip_rect.w = gv.w-1;
	clip_rect.h = gv.h-1;
	SDL_SetClipRect(screen, &clip_rect);

	// draw terrain

	RGB rgb_GROUND(100,42,42);
	RGB rgb_WATER(0,0,128);
	RGB rgb_PLATEAU(122, 82, 48);

	if (gv.mx < 0 || gv.my < 0) {
		SDL_Rect rect;
		rect.x = rect.y = 0; rect.w = gv.w; rect.h = gv.h;
		SDL_FillRect(screen, &rect, 0);
	}

	sint4 tx0 = gv.mx/tile_pixels;
	sint4 ty0 = gv.my/tile_pixels;  
	if (tx0 < 0) tx0 = 0;
	if (ty0 < 0) ty0 = 0;  
	sint4 x0  = tx0 * tile_pixels - gv.mx + gv.x;
	sint4 y0  = ty0 * tile_pixels - gv.my + gv.y;
	sint4 txn = (gv.w+gv.mx-tx0*tile_pixels+tile_pixels-1)/tile_pixels;
	sint4 tyn = (gv.h+gv.my-ty0*tile_pixels+tile_pixels-1)/tile_pixels;

	txn = ::min(txn, game->get_map().get_width());
	tyn = ::min(tyn, game->get_map().get_height());  

#if 0
	cout << "XXXXXXXXXXX" << endl;
	cout << gv.mx << " " << gv.my << e
		ndl;
	cout << gv.w << " " << gv.h << endl;
	cout << tx0 << " " << ty0 << endl;
	cout << txn << " " << tyn << endl;
	cout << x0 << " " << y0 << endl;    
#endif

	RGB black(0,0,0);

	FORS (ty, tyn) {

		const GameTile *p = &game->get_map()(tx0, ty+ty0);

		sint4 x = x0;
		FORS (tx, txn) {

			SDL_Rect rect;

			Tile::Split split = p->get_split();

			sint4 c = p->get_typeW();
			uint4 col;
			if (c == GROUND)     col = p->get_min_h() == 0 ? rgb_GROUND.color() : rgb_PLATEAU.color();
			else if (c == WATER) col = rgb_WATER.color();
			else                 col = black.color();

			rect.x = x; rect.y = y0; rect.h = rect.w = tile_pixels;
			SDL_FillRect(screen, &rect, col);

			if (split != Tile::NO_SPLIT) {

				int tp = tile_pixels+1;
				int tx, ty, dx, dy;
				if (split == Tile::TB_SPLIT) {
					tx = x+tile_pixels;
					ty = y0+tile_pixels;
					dx = -tp;
					dy = -tp;
				} else {
					tx = x+tile_pixels;
					ty = y0;
					dx = -tp;
					dy = tp;
				}

				sint4 c = p->get_typeE();
				uint4 col;
				if (c == GROUND)     col = p->get_min_h() == 0 ? rgb_GROUND.color_alpha() : rgb_PLATEAU.color_alpha();
				else if (c == WATER) col = rgb_WATER.color_alpha();
				else                 col = black.color_alpha();

				filledRTriangle(screen, tx, ty, dx, dy, col);
			}

			if (DISP_GRID)
				rectangleColor(screen,
						x, y0, x + tile_pixels, y0 + tile_pixels,
						black.color_alpha());

			// views

			if (DISP_KNOW && tile_pixels > game->get_player_num()*2+2) {

				FORS (i, game->get_player_num()) {
					if (game->get_player_info(i).explored.is_set(tx+tx0, ty+ty0)) {
						rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2;
						rect.y = y0+tile_pixels/2;
						rect.x -= 2;
						rect.y -= 2;
						rect.h = rect.w = 6;
						SDL_FillRect(screen, &rect, colors[i+4].color());
					}
				}
			}

			if (DISP_VISI && tile_pixels > game->get_player_num()*2+2) {

				// draw colored 2x2 blocks each player who sees tile

				FORS (i, game->get_player_num()) {
					if (game->get_map()(tx, ty).get_split() != Tile::NO_SPLIT) {
						if (game->get_player_info(i).visible_W.is_set(tx+tx0, ty+ty0)) {
							rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2 - 2;
							rect.y = y0+tile_pixels/2;
							rect.h = rect.w = 2;
							SDL_FillRect(screen, &rect, colors[i].color());
						}
						if (game->get_player_info(i).visible_E.is_set(tx+tx0, ty+ty0)) {
							rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2 + 2;
							rect.y = y0+tile_pixels/2;
							rect.h = rect.w = 2;
							SDL_FillRect(screen, &rect, colors[i].color());
						}
					} else {
						if (game->get_player_info(i).visible_W.is_set(tx+tx0, ty+ty0)) {
							rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2;
							rect.y = y0+tile_pixels/2;
							rect.h = rect.w = 2;
							SDL_FillRect(screen, &rect, colors[i].color());
						}
					}
				}
			}

			++p;
			x += tile_pixels;
		}
		y0 += tile_pixels;
	}

	// draw objects

	FORS (i, game->get_player_num()+1) {

		const RGB &rgb = colors[i];

		FORALL(game->get_objs(i), it) {

			GameObj *gob = (*it)->get_GameObj();
			if (gob == 0) continue;

			if (!gob->has_attr("shape")) continue;

			if (gob->get_int("shape") == Object::CIRCLE) {

				// check whether object is visible
				sint4 cx = scale(gob->get_int("x"), gv.mag);
				sint4 cy = scale(gob->get_int("y"), gv.mag);
				sint4 r  = scale(gob->get_int("radius"), gv.mag);
				if (cx - r > gv.mx+gv.w) continue;
				if (cx + r < gv.mx) continue;
				if (cy - r > gv.my+gv.h) continue;
				if (cy + r < gv.my) continue;
				//      SDL_Rect rect;
				//      rect.x = cx-gv.mx; rect.y = cy-gv.my; rect.h = rect.w = 2*r;
				// SDL_FillRect(screen, &rect, rgb.color());    
				filledCircleColor(screen, cx-gv.mx, cy-gv.my, r, rgb.color_alpha());

				if (DISP_SEEN) {
					FORS (j, game->get_player_num()) {
						if (j != i && gob->sod.seen_by.is_set(j)) {
							const RGB black(0,0,0);
							circleColor(screen, cx-gv.mx, cy-gv.my, 3, black.color_alpha());
							break;
						}
					}
				}

				sintptr *m = gob->get_int_ptr("marker");
				if (m)
					draw_marker(cx-gv.mx, cy-gv.my, r, *m);
				else
					draw_marker(cx-gv.mx, cy-gv.my, r, MARKER_NO);

				if (gob == obj1 || gob == obj2) {
					RGB white(255,255,255);
					circleColor(screen, cx-gv.mx, cy-gv.my, r, white.color_alpha());
				}
			} else if (gob->get_int("shape") == Object::RECTANGLE) {

				// check whether object is visible
				sint4 cx = scale(gob->get_int("x"), gv.mag);
				sint4 cy = scale(gob->get_int("y"), gv.mag);
				sint4 w  = scale(gob->get_int("width"), gv.mag);
				sint4 h  = scale(gob->get_int("height"), gv.mag);
				sint4 r  = scale(gob->get_int("radius"), gv.mag);

				sint4 x1 = cx - w/2;
				sint4 x2 = cx + w/2;
				sint4 y1 = cy - h/2;
				sint4 y2 = cy + h/2;

				if (x1 > gv.mx+gv.w) continue;
				if (x2 < gv.mx) continue;
				if (y1 > gv.my+gv.h) continue;
				if (y2 < gv.my) continue;

				lineColor(screen, x1-gv.mx, y1-gv.my, x2-gv.mx, y1-gv.my, rgb.color_alpha());
				lineColor(screen, x1-gv.mx, y2-gv.my, x2-gv.mx, y2-gv.my, rgb.color_alpha());

				lineColor(screen, x1-gv.mx, y1-gv.my, x1-gv.mx, y2-gv.my, rgb.color_alpha());
				lineColor(screen, x2-gv.mx, y1-gv.my, x2-gv.mx, y2-gv.my, rgb.color_alpha());

				//	SDL_Rect rect;
				//	rect.x = cx-gv.mx; rect.y = cy-gv.my; rect.h = h; rect.w = w;
				//	SDL_FillRect(screen, &rect, rgb.color());    

				//	filledCircleColor(screen, cx-gv.mx, cy-gv.my, r, rgb.color_alpha());

				if (DISP_SEEN) {
					FORS (j, game->get_player_num()) {
						if (j != i && gob->sod.seen_by.is_set(j)) {
							const RGB black(0,0,0);
							circleColor(screen, cx-gv.mx, cy-gv.my, 3, black.color_alpha());
							break;
						}
					}
				}

				sintptr *m = gob->get_int_ptr("marker");
				if (m)
					draw_marker(cx-gv.mx, cy-gv.my, r, *m);
				else
					draw_marker(cx-gv.mx, cy-gv.my, r, MARKER_NO);

				if (gob == obj1 || gob == obj2) {
					RGB white(255,255,255);
					circleColor(screen, cx-gv.mx, cy-gv.my, r, white.color_alpha());
				}
			} else if (gob->get_int("shape") == Object::LINE) {

				sint4 x1 = scale(gob->get_int("x1"), gv.mag);
				sint4 x2 = scale(gob->get_int("x2"), gv.mag);
				sint4 y1 = scale(gob->get_int("y1"), gv.mag);
				sint4 y2 = scale(gob->get_int("y2"), gv.mag);

				lineColor(screen, x1-gv.mx, y1-gv.my, x2-gv.mx, y2-gv.my, rgb.color_alpha());
			}
		}
	}

	tx0 = gv.mx/tile_pixels;
	ty0 = gv.my/tile_pixels;  
	if (tx0 < 0) tx0 = 0;
	if (ty0 < 0) ty0 = 0;  
	x0  = tx0 * tile_pixels - gv.mx + gv.x;
	y0  = ty0 * tile_pixels - gv.my + gv.y;
	txn = (gv.w+gv.mx-tx0*tile_pixels+tile_pixels-1)/tile_pixels;
	tyn = (gv.h+gv.my-ty0*tile_pixels+tile_pixels-1)/tile_pixels;

	txn = ::min(txn, game->get_map().get_width());
	tyn = ::min(tyn, game->get_map().get_height());

	FORS (ty, tyn) {

		sint4 x = x0;
		FORS (tx, txn) {

			SDL_Rect rect;
			rect.x = x; rect.y = y0; rect.h = rect.w = tile_pixels;

			// views

			if (DISP_KNOW && tile_pixels > game->get_player_num()*2+2) {

				FORS (i, game->get_player_num()) {
					if (game->get_player_info(i).explored.is_set(tx+tx0, ty+ty0)) {
						rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2;
						rect.y = y0+tile_pixels/2;
						rect.x -= 2;
						rect.y -= 2;
						rect.h = rect.w = 6;
						SDL_FillRect(screen, &rect, colors[i+4].color());
					}
				}
			}

			if (DISP_VISI && tile_pixels > game->get_player_num()*2+2) {

				// draw colored 2x2 blocks each player who sees tile

				FORS (i, game->get_player_num()) {

					if (game->get_map()(tx,ty).get_split() != Tile::NO_SPLIT) {

						if (game->get_player_info(i).visible_W.is_set(tx+tx0, ty+ty0)) {
							rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2 - 2;
							rect.y = y0+tile_pixels/2;
							rect.h = rect.w = 2;
							SDL_FillRect(screen, &rect, colors[i].color());
						}
						if (game->get_player_info(i).visible_E.is_set(tx+tx0, ty+ty0)) {
							rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2 + 2;
							rect.y = y0+tile_pixels/2;
							rect.h = rect.w = 2;
							SDL_FillRect(screen, &rect, colors[i].color());
						}

					} else {
						if (game->get_player_info(i).visible_W.is_set(tx+tx0, ty+ty0)) {
							rect.x = x + tile_pixels/2 - (game->get_player_num()/2) * 2 + i * 2;
							rect.y = y0+tile_pixels/2;
							rect.h = rect.w = 2;
							SDL_FillRect(screen, &rect, colors[i].color());
						}
					}
				}
			}

			x += tile_pixels;
		}
		y0 += tile_pixels;
	}


	const RGB rgb(255,255,255);

	FORALL(game->get_boundaries(), it) {

		GameObj *gob = (*it)->get_GameObj();
		if (gob == 0) continue;

		gob->update_points();
		sint4 x  = scale(*gob->sod.x1, gv.mag);
		sint4 y  = scale(*gob->sod.y1, gv.mag);      
		sint4 x2 = scale(*gob->sod.x2, gv.mag);
		sint4 y2 = scale(*gob->sod.y2, gv.mag);      

		// cout << x << " " << y << " " << x2 << " " << y2 << endl;
		if ((x > gv.mx+gv.w || x < gv.mx || y > gv.my+gv.h || y < gv.my) &&
				(x2 > gv.mx+gv.w || x2 < gv.mx || y2 > gv.my+gv.h || y2 < gv.my))
			continue;

		lineColor(screen, x-gv.mx, y-gv.my, x2-gv.mx, y2-gv.my, rgb.color_alpha());
	}

	/*
		FORALL(game->get_dead_objs(), it) {

		GameObj *gob = (*it)->get_GameObj();
		if (gob == 0) continue;

		if (*gob->sod.owner == -1) {
		sint4 cx = scale(gob->get_int("x"), gv.mag);
		sint4 cy = scale(gob->get_int("y"), gv.mag);
		sint4 r  = scale(gob->get_int("radius"), gv.mag);

		RGB white(255,255,255);
		circleColor(screen, cx-gv.mx, cy-gv.my, r, white.color_alpha());
		}
		}
		*/
}
#endif 
//===================================================================
