#include "Game.H"
#include "GameObj.H"

#include "Sheep.H"
#include "Mineral.H"
#include "Obstacle.H"
#include "ControlCenter.H"
#include "AIGlobal.H"
#include "AIEventHandler.H"
#include "AIState.H"
#include "AIObject.H"
#include "Worker.H"
#include "Fighter.H"
#include "Graph.H"

using namespace std;

namespace bzAI {

uint4 player_bitmap;
uint4 enemy_bitmap;
uint4 neutral_bitmap;
uint4 all_bitmap;

sint4 distance_cmp(GameObj* g1, GameObj* g2, sint4 cmp){
	sint4 d = distance2(Loc(*g1->sod.x,*g1->sod.y),Loc(*g2->sod.x, *g2->sod.y));
	int r1 = 0 , r2 = 0;
	if (*g1->sod.shape == Object::CIRCLE)
		r1 = *g1->sod.radius; else
		r1 = 0*(*g1->sod.x2 - *g1->sod.x1)/2;
	if (*g2->sod.shape == Object::CIRCLE)
		r2 = *g2->sod.radius; else
		r2 = 0*(*g2->sod.x2 - *g1->sod.x1)/2;

	cmp += *g1->sod.radius + *g2->sod.radius;
	if (d <= cmp*cmp) return 1; else
		return -1;
}

sint4 AIObject :: getId(){
	static PlayerInfo& my_info=AIEventHandler :: handler->state->gsm->get_game().get_cplayer_info();
	return my_info.get_id(gob);
}



AIObject :: AIObject(GameObj* o, real8 _weight):weight(_weight), gob(o), isDead(false), location_cache(-1,-1){
	const Game *game = &HANDLER->state->gsm->get_game();
	if (*gob->sod.owner==-1)
		owner_cache = game->get_player_num();
	else owner_cache = *gob->sod.owner;
	assert(o);
}
AIObject :: AIObject():weight(1), gob(NULL), location_cache(-1,-1){}


void AIObject::cacheAttributes(){
	gob->update_sod();
	location_cache = Loc(*gob->sod.x, *gob->sod.y);
}

int AIObjectCollector :: count(){
	int ret=0;
	REP(i,32) 
		ret+=v[i].size;
	return ret;
}

AIObject* AIObjectCollector :: createAIObject(GameObj* gob){
	if (gob->bp_name() == "sheep") return new Sheep(gob); 
	if (gob->bp_name() == "mineral") return new Mineral(gob);
	if (gob->bp_name() == "controlCenter") return new ControlCenter(gob);
	if (gob->bp_name() == "worker") return new Worker(gob);
	if (gob->bp_name() == "marine") return new Fighter(gob);
	if (gob->bp_name() == "tank") return new Fighter(gob);

	cout << "createAIObject : unrecognized gob - " << gob->bp_name() << endl;
//	ERR(gob->bp_name() + " : cannot associate object");
	return NULL;
}



void AIObjectCollector :: addAIObject(AIObject*o){
	assert(o);
	v[o->getOwner()].push_back(o);
	if (INSTANCEOF(o,Fighter))
		fighters[o->getOwner()].push_back(o);
//	std :: cout << "adding object "<<v.size()<<":" << o->gob->bp_name() << " owner "<< o->getOwner() << std :: endl;
}

AIObject* AIObjectCollector :: getAIObject(Loc l){
	return loc2aiob[l];
}

#define SET_TEMPLATE(T) \
template T* AIObjectCollector :: getFirst<T>(uint4, Region);  \
template T* AIObjectCollector :: getNext<T>(AIObject*, uint4, Region) 

SET_TEMPLATE(Mineral);
SET_TEMPLATE(Worker);
SET_TEMPLATE(ControlCenter);
SET_TEMPLATE(Obstacle);
SET_TEMPLATE(AIObject);
SET_TEMPLATE(Agent);
//SET_TEMPLATE(Sheep);
SET_TEMPLATE(Fighter);


template <class T>
aiob_vector* getVector(aiob_vector* v, aiob_vector* fighters){
	return v;
}

template <>
aiob_vector* getVector<Fighter>(aiob_vector* v, aiob_vector* fighters){
	return fighters;
}




template <class T>
T* AIObjectCollector::getFirst(uint4 cid_bitmap, Region r){
	iter_cache=-1;
	aiob_vector* v = getVector<T>(this->v, this->fighters);
	for (int i=0,bi=1;i<32 && cid_bitmap;++i,bi<<=1){
		if (cid_bitmap & bi){
			REP(j, v[i].size){
				if (INSTANCEOF(v[i].t[j],T)) { 
					iter_cache = j; 
					index_cache = i;
//					cout << "GetFirst returning "<<v[i][j]->gob->bp_name()<<" "<<Loc(*v[i][j]->gob->sod.x,*v[i][j]->gob->sod.y)<<endl;
					return (T*)v[i].t[j]; 
				}
			}
			cid_bitmap^=bi;
		}
	}
//	cout << "AIObjectCollector :: getFirst - nothing found" << endl;
	return NULL;
}


template <class T>
T* AIObjectCollector :: getNext(AIObject* o, uint4 cid_bitmap, Region r){
	//assert(o);
	aiob_vector* v = getVector<T>(this->v, this->fighters);
	bool was=false;
	for (int i=((iter_cache>-1 && v[index_cache].t[iter_cache] == o) ? index_cache : 0),bi=1<<i;i<32 && cid_bitmap;++i,bi<<=1){
		if (cid_bitmap & bi){
			FORL(j,((iter_cache>-1) && v[index_cache].t[iter_cache] == o) ? iter_cache : 0, v[i].size-1){
				if (iter_cache>-1 && v[index_cache].t[iter_cache] == o) iter_cache=-1;
				//assert(it);
				if (was && INSTANCEOF(v[i].t[j],T)) { 
					iter_cache = j; 
					index_cache = i;
//					cout << "GetNext returning "<<v[i][j]->gob->bp_name()<<" "<<Loc(*v[i][j]->gob->sod.x,*v[i][j]->gob->sod.y)<<endl;
					return (T*)v[i].t[j]; 
				}
				if (v[i].t[j] == o) was=true;
			}
			cid_bitmap^=bi;
		}
	}
//	cout << "AIObjectCollector :: getNext - nothing found:" << o->gob->bp_name() <<endl;
	
	return NULL;
}



void AIObjectCollector :: removeGameObj(GameObj* gob){
	assert(gob);
	REP(i,32)
	REP(j, v[i].size){
		if (v[i].t[j]->gob==gob){
			v[i].t[j]->isDead=true;
			v[i].remove(v[i].t[j]);
			break;
		}
	}
	REP(i,32)
	REP(j, fighters[i].size){
		if (fighters[i].t[j]->gob==gob){
			fighters[i].t[j]->isDead=true;
			fighters[i].remove(fighters[i].t[j]);
			return;
		}
	}

}

void aiob_vector::push_back(AIObject* o){
	t[size++]=o;
}
void aiob_vector::remove(AIObject* o){
	REP(i,size)
		if (t[i]==o){
			t[i]=t[--size];
			return;
		}
}

aiob_vector::aiob_vector(){
	size=0;
}

}
