#ifndef AIObject_H
#define AIObject_H

#include <vector>

#include "Global.H"
#include "AIGlobal.H"
#include "Region.H"
class GameObj;

// var_name, class_name, cid_bitmap, region
#define FOR_OBJ_OF(o, t, bm, r) for (t* o = dynamic_cast<t*>(HANDLER->collector->getFirst<t>(bm, r)); o; o = dynamic_cast<t*>(HANDLER->collector->getNext<t>(o, bm, r)))

// var_name, class_name, cid_bitmap
#define FOR_EACH_OBJ_OF(o, t, bm) FOR_OBJ_OF(o, t, bm, whole_map)

#define FOR_OBJ(o, t, r) FOR_OBJ_OF(o, t, all_bitmap, r)
#define FOR_EACH_OBJ(o, t) FOR_OBJ(o, t, whole_map)

namespace bzAI {
	extern uint4 player_bitmap;
	extern uint4 enemy_bitmap;
	extern uint4 neutral_bitmap;
	extern uint4 all_bitmap;

	sint4 distance_cmp(GameObj*, GameObj*, sint4);

	class AIObject {
		protected:
		AIObject();
		real8 weight;
		Loc location_cache;
		sint4 owner_cache;
		public:
		bool isDead;
		GameObj *gob;
		sint4 getId();
		inline sint4 getOwner(){
			return owner_cache;
		}
		void cacheAttributes();
		Loc& getLoc(){
//			assert(location_cache.x);
			return location_cache;
		}

		virtual void setAction(){}
		inline real8 getWeight(){ return weight; }
		AIObject(GameObj* o, real8 weight = 1);
		virtual ~AIObject(){}
	};

	struct aiob_vector {
		AIObject* t[10000];
		int size;
		void push_back(AIObject*);
		void remove(AIObject*);
		aiob_vector();
	};
	
	class AIObjectCollector {
		aiob_vector v[32];
		aiob_vector fighters[32];
		std::map<Loc, AIObject*> loc2aiob;
		int index_cache, iter_cache;

		public:
		static AIObject* createAIObject(GameObj*);
		
		void addAIObject(AIObject*);
		AIObject* getAIObject(Loc l);
		void removeGameObj(GameObj*);

		int count();

		template <class T>
		T* getFirst(uint4, Region=whole_map);
		template <class T>
		T* getNext(AIObject*, uint4, Region=whole_map);

	};

}

#endif
