#include <iostream>

#include "Global.H"
#include "GameObj.H"
#include "Distance.H"
#include "Agent.H"
#include "Path.H"
#include "GameConst.H"
#include "AIEventHandler.H"
#include "Distance.H"
#include "Sheep.H"
#include "Mineral.H"
#include "Configuration.H"

using namespace std;
namespace bzAI {

	Agent :: Agent(GameObj* gob) : AIObject(gob), path(new Path), locTarget(-sint4_inf, -sint4_inf), rotation_dir(0.3), aiTarget(NULL),
	path_search_delayed(false), groups(NULL){
		assert(gob);
	}

	vector<Agent*>& Agent::getGroup(){
		return groups[group_id];
	}

	uint4 Agent :: st_norm(){
		return *gob->sod.max_speed;
	}


	bool Agent :: reachedEnd(){
		assert(path);
		return path->reachedEnd();
	}

	void Agent :: addInteractionForce(Force f){
		//	DENTER();
		interactionForce += f;
		//	PDEBUG("Adding force "<<f.x<<' '<<f.y);
		//	DEXIT();
	}
	void Agent :: addIntentionForce(Force f){
		intentionForce += f;
	}

	Loc Agent :: getTarget(){
		assert(path);
		return path->beforeLast();
	}

	void Agent :: moveTo(Loc to, uint4 speed){
		if (dont_move || to == getLoc()){
			return;
		}
		Vector<sint4> params;
		//DENTER();

		//Loc myLoc = getLoc();
		//cout << "difference : (" << to.x - myLoc.x << ", " << to.y - myLoc.y << ") " << endl;
		params.push_back(to.x);
		params.push_back(to.y);
		params.push_back(speed);
		gob->set_action("move", params);
		//DEXIT();
	}

	void Agent :: gotoTarget(AIObject* aiob){
		if (aiTarget==aiob && path_update_count>0) return;
		assert(aiob);
		gotoTarget(aiob->getLoc());
		aiTarget=aiob;
		path_update_count=HANDLER->config->getInt("agent_path_update_time");
	}

	void Agent :: gotoTarget(Loc newLoc){
		aiTarget=NULL;
		if (newLoc.distance(locTarget) >= dist_close || !path || path->empty() ) // TODO nie zepsulem path_search_Delayed?
			setTarget(newLoc);
		assert(path || path_search_delayed);


	}

	static bool path_search_ok(int id){
		static int delay = HANDLER->config->getInt("agent_path_search_delay");
		return id % delay == HANDLER->current_frame % delay;
	}

	void Agent :: setTarget(Loc newLoc){
		locTarget = newLoc;
		delete path;
		path = NULL;
		if (!path_search_ok(getId())){
			path_search_delayed = true;
			path = new Path();
			return;
		} else path_search_delayed = false;
		// cout << "searching path"<<endl;
		path = new Path( getLoc(), locTarget);
	}
	// sets the agent rotation depending on position in the group and movement direction
	void Agent :: setRotation(vec2 dir){
		if (groups.size()==0) return;
		vector<Agent*>& group = getGroup();
		dir = dir.normalize(1.0);

		int right_hand = 0;
		REP(i, group.size()){
			if (-group[i]->getLoc().x * dir.y + group[i]->getLoc().y * dir.x > -getLoc().x * dir.y + getLoc().y * dir.x) 
				++right_hand;
		}
		if (right_hand > group.size()/2) rotation_dir = 1; else rotation_dir = -1;

	}
	void Agent :: setMovement(){
		if (path->empty() && path_search_delayed && path_search_ok(getId())){
			path = new Path( getLoc(), locTarget);
			path_search_delayed = false;
		}
		if (aiTarget!=NULL&& path_update_count--<0 ) gotoTarget(aiTarget);
		if (path)
			path->updatePos(getLoc());
		static real8 fmul1=HANDLER->config->getReal8("force_multiplier");
		static real8 edist=HANDLER->config->getReal8("enemy_distance");

		if (!path->empty()){
			setRotation(vec2(path->back().x-getLoc().x,path->back().y - getLoc().y));
		}

		if (!path->empty() && getLoc().distance(locTarget)>=dist_close)
			addIntentionForce(Force(getLoc(), path->getTarget()).normalize(st_norm()));
		/*FOR_EACH_OBJ(o,Sheep) addInteractionForce(interactionFrom(o,fmul1));
		  FOR_EACH_OBJ(o,Obstacle) addInteractionForce(interactionFrom(o,fmul1));
		  FOR_EACH_OBJ_OF(o,Agent,player_bitmap) addInteractionForce(interactionFrom(o,fmul1));
		  FOR_EACH_OBJ(o,Mineral) addInteractionForce(interactionFrom(o,fmul1));*/

		FOR_EACH_OBJ(o,AIObject) 
			if (o!=aiTarget) 
				addInteractionForce(interactionFrom(o,fmul1));

		//	FOR_EACH_OBJ_OF(o,Agent,enemy_bitmap) addInteractionForce(interactionFrom(o,(3.0+edist*edist)*getWeight()/o->getWeight()));

		//	cout << "Testing getLoc: "<<getLoc()<<" "<<gob->bp_name()<<endl;
		moveTo(getLoc()+ (interactionForce + intentionForce)*10.0, st_norm());
		interactionForce = Force(0, 0);
		intentionForce = Force(0, 0);
	}



	Force Agent :: interactionFrom(AIObject* aiob, real8 fmul){
		static real8 force_max_distance = HANDLER->config->getInt("force_max_distance");
		if (Distance::distance(*gob, *aiob->gob)> force_max_distance
				|| aiob==this) return Force(0,0);
		real8 alf,d,a,b,x,y;
		if (aiob->gob->has_attr("is_mobile") && aiob->gob->get_int("is_mobile")){
			alf=GameConst::angle_from_dir(aiob->gob->get_int("heading"),GameConst::HEADING_N);
			alf/=180.0;
			alf*=M_PI;
			d=1.5*(aiob->gob->get_int("max_speed"));
		} else {d=alf=0;}
		alf+=rrand()-0.5;
		x=getLoc().x-aiob->getLoc().x;
		y=getLoc().y-aiob->getLoc().y;

		real8 sin_alf = sin(alf), cos_alf = cos(alf), cos_malf = cos_alf, sin_malf = -sin_alf;

		real8 x1=x*cos_malf-y*sin_malf,y1=x*sin_malf+y*cos_malf;
		x=x1;
		y=y1;
		x-=d;
		a=(Loc(0,0).distance(Loc((sint4)(x+d),(sint4)y))+Loc(0,0).distance(Loc((sint4)(x-d),(sint4)y)))/2;
		if (a*a-d*d<=0) return Force(rrand(),rrand());
		b=sqrt(a*a-d*d);

		x*=b/a;
		y*=a/b;

		x+=rrand();
		y+=rrand();


		x+=d;
		x1=x*cos_alf-y*sin_alf;y1=x*sin_alf+y*cos_alf;
		x=x1;
		y=y1;

		Force f_rand;
		if (INSTANCEOF(aiob,Agent)){
			f_rand = Force(-y,x);
			if (((Agent*)aiob)->dont_move){
				f_rand*=(2.0*rotation_dir+rrand()-0.5);
			} else f_rand*=(0.5 + 0.5 * rrand());
		}

		return (Force(x,y)+f_rand).normalize(fmul*st_norm()/(4.0+sqr(Distance::distance(*gob,*aiob->gob)))) * (aiob->getWeight() / getWeight());
	}

	void Agent :: stop(){
		//DENTER();
		delete path;
		path = new Path();
		dont_move = true;
		//moveTo(Loc(getLoc().x-1,getLoc().y),1);
		gob->set_action("stop",Vector<sint4>());
		//DEXIT();
	}

}


