#include <set>
#include <map>

#include "SType.H"
#include "Options.H"

#include "AIEventHandler.H"
#include "Coordinator.H"
#include "AIState.H"
#include "Graph.H"
#include "Mineral.H"
#include "ControlCenter.H"
#include "Path.H"
#include "Fighter.H"
#include "Formation.H"
#include "Configuration.H"

using namespace bzAI;
using namespace std;
Coordinator :: Coordinator():workerC(NULL),fighterC(NULL){
	if (HANDLER->game_type == GAME_1)
		workerC=new WorkerCoordinator();
	if (HANDLER->game_type == GAME_2)
		fighterC=new Game2FighterCoordinator();
	if (HANDLER->game_type == GAME_4)
		fighterC=new Game4FighterCoordinator();

}
Coordinator :: ~Coordinator(){
	delete workerC;
	delete fighterC;
}

void Coordinator :: setAction(){
	switch (HANDLER->game_type){
		case GAME_1: workerC->setAction(); break;
		case GAME_2: fighterC->setAction(); break;
		case GAME_3: break;
		case GAME_4: fighterC->setAction(); break;
	}
}

real8 kryterium1(AIObject *o1, AIObject *o2){
	return Game::distance(*o1->gob,*o2->gob);
}

real8 kryterium2(AIObject *o1, AIObject *o2){
	return o2->gob->get_int("hp");
}

real8 kryterium3(AIObject *o1, AIObject *o2){
	return kryterium1(o1,o2)+kryterium2(o1,o2)/2;
}

// odleglosc punktu od prostej zadanej wektorem kierunkowym v i punktem s
static real8 line_distance(vec2 s, vec2 v, vec2 p){
	return (v.y * p.x - v.x * p.y - v.y * s.x + v.x * s.y)/v.norm();
}

void Game2FighterCoordinator :: initialize(){}

void getGroups(vector<vector<Agent*> >& ret, uint4 bitmap, int group_dist){
	bool out[10000]={0};
	FOR_EACH_OBJ_OF(aiob, Agent, bitmap){
		if (out[aiob->getId()]) continue;
		queue<Agent*> q;
		ret.push_back(vector<Agent*>());
		q.push(aiob);
		out[aiob->getId()] = true;
		while (!q.empty()){
			Agent* f = q.front();
			ret.back().push_back(f);
			q.pop();
			FOR_EACH_OBJ_OF(cand, Fighter, bitmap)
				if (!out[cand->getId()] && distance2(cand->getLoc(),f->getLoc()) <= group_dist * group_dist){
					out[cand->getId()] = true;
					q.push(cand);
				}
		}
	}
	REP(j,ret.size())
		REP(i,ret[j].size()){
			ret[j][i] -> groups = ret;
			ret[j][i] -> group_id = j;
		}
}

void Game2FighterCoordinator :: setAction(){
	const Game *game = &HANDLER->state->gsm->get_game();
  	const sint4 cid = game->get_client_player(); 
	static int counter=0;
	++counter;
	static real8 midx=0,midy=0, enemy_midx=0, enemy_midy=0;


	static real8 enemy_dist=1000000;
	vec2 direction;
	static real8 enemy_spread=0;


	int t=0;
	static int player_group_dist = HANDLER->config->getInt("player_group_dist");
	static int enemy_group_dist = HANDLER->config->getInt("enemy_group_dist");
	vector<vector<Agent*> > player_groups;
	getGroups(player_groups, player_bitmap, player_group_dist);
	vector<vector<Agent*> > enemy_groups;
	getGroups(enemy_groups, enemy_bitmap, enemy_group_dist);
	vector<Loc> enemy_group_pos;

	int max_enemy_group = 0;
	
	REP(i,enemy_groups.size()){
		Loc middle(0,0);
		if (enemy_groups[i].size() > enemy_groups[max_enemy_group].size())
			max_enemy_group = i;
		REP(j,enemy_groups[i].size()){
			middle.x+=enemy_groups[i][j]->getLoc().x;
			middle.y+=enemy_groups[i][j]->getLoc().y;
		}
		middle.x/=enemy_groups[i].size();
		middle.y/=enemy_groups[i].size();

		enemy_group_pos.push_back(middle);
	}


	FOR_EACH_OBJ_OF (aiob, Fighter,player_bitmap){
		real8 min_dist=1000000000.0,tmp;
		AIObject* enemy_ob=NULL;
		FOR_EACH_OBJ_OF(enemy_it, Agent, enemy_bitmap){
			if ((enemy_it->getGroup().size()>=3 || max_enemy_group<3)&&(tmp=kryterium1(aiob,enemy_it))<min_dist){
				min_dist=tmp;
				enemy_ob=enemy_it;
			}
		}
		if (enemy_ob == NULL || min_dist>200){
			min_dist=10000000.0;
//			cout <<"Going for control center"<<endl;
			FOR_EACH_OBJ_OF(enemy_it, ControlCenter, enemy_bitmap)
				if ((tmp=kryterium1(aiob,enemy_it)) < min_dist){
					min_dist = tmp;
					enemy_ob = enemy_it;
				}
		}
		if (enemy_ob!=NULL) aiob->gotoTarget(enemy_ob);
	}

	FOR_EACH_OBJ_OF(aiob,Fighter,player_bitmap) {
		aiob->setAction();

	}
	FOR_EACH_OBJ_OF(aiob,Fighter,player_bitmap) {
		aiob->setMovement();
	}
}

void Game4FighterCoordinator :: setAction(){
	const Game *game = &HANDLER->state->gsm->get_game();
  	const sint4 cid = game->get_client_player(); 
	static int counter=0;
	++counter;
	static real8 midx=0,midy=0, enemy_midx=0, enemy_midy=0;

//	static WingsFormation* form=new WingsFormation(HANDLER->config->getInt("fighter_formation_dist"),25,0);
//	static Formation* form=new LineFormation(HANDLER->config->getInt("fighter_formation_dist"));
	static EnemyFollowY* form=new EnemyFollowY();



	int howMany;
	if (counter==1){
		FOR_EACH_OBJ_OF(aiob,Fighter,player_bitmap) form->addAgent(aiob);	
		FOR_EACH_OBJ_OF(enob,Fighter,enemy_bitmap) form->addEnemy(enob);
		howMany=0;
		midx=midy=0;
		FOR_EACH_OBJ_OF(aiob,Fighter,player_bitmap) { 
                  form->addAgent(aiob); 
                  midx+=aiob->getLoc().x;
                  midy+=aiob->getLoc().y;
                  ++howMany;
                }
		FOR_EACH_OBJ_OF(enob,Fighter,enemy_bitmap) form->addEnemy(enob);
		if (howMany>0){
			midx/=howMany;
			midy/=howMany;
		}
	}

	howMany=0;
	enemy_midx=enemy_midy=0;
	FOR_EACH_OBJ_OF(enob,Fighter,enemy_bitmap) { enemy_midx+=enob->getLoc().x;enemy_midy+=enob->getLoc().y;++howMany;}
	if (howMany>0){
		enemy_midx/=howMany;
		enemy_midy/=howMany;
	}


	static real8 enemy_dist=1000000;
	vec2 direction;
	static real8 enemy_spread=0;

	direction = (vec2(enemy_midx,enemy_midy) - vec2(midx,midy)).normalize(1.0);
	if (counter%10==1)
	FOR_EACH_OBJ_OF(aiob,Fighter,player_bitmap)
		FOR_EACH_OBJ_OF(enob,Fighter,enemy_bitmap){
			enemy_dist<?=Game::distance(*aiob->gob,*enob->gob);
			enemy_spread >?= line_distance(vec2(midx,midy), direction, vec2(enob->getLoc().x, enob->getLoc().y));
		}

	if (enemy_dist>100) {
		midx += direction.x;
		midy += direction.y;
		form->createFormation(Loc(midx,midy),direction);


	} else  {

		int t=0;
		int path_delay=1;
		FOR_EACH_OBJ_OF (aiob, Fighter,player_bitmap){
//			aiob->gotoTarget(Loc(0,0));
			t=(t+1)%path_delay;
			if (!aiob->hasTarget() || counter%path_delay==t) {
				real8 min_dist=1000000000.0,tmp;
				AIObject* enemy_ob=NULL;
				FOR_EACH_OBJ_OF(enemy_it, AIObject, enemy_bitmap){
					if ((tmp=kryterium1(aiob,enemy_it))<min_dist){
						min_dist=tmp;
						enemy_ob=enemy_it;
					}
				}
				//			assert(enemy_ob);
				if (enemy_ob!=NULL) aiob->gotoTarget(enemy_ob);
			}
		}
	}

	FOR_EACH_OBJ_OF(aiob,Fighter,player_bitmap) {
		aiob->setAction();
		aiob->setMovement();

	}

}



void WorkerCoordinator :: setAction(){
	const Game *game = &HANDLER->state->gsm->get_game();
  	const sint4 cid = game->get_client_player(); 
        static sint4 mineralWorkersMax = HANDLER->config->getInt("mineral_workers_max");
	set<Loc> mineralsSet;
	map<Mineral*, int> used;
        // TODO zbior mineralow jest za kazda tura zbierany od nowa	
	FOR_EACH_OBJ (aiob, Mineral) 
          if (aiob->workersNum < mineralWorkersMax) mineralsSet.insert(aiob->getLoc());
	FOR_EACH_OBJ_OF (aiob, Worker,player_bitmap) {
		if (aiob->getOwner()==cid) {

			bool attr;
			Options :: get("-attr",attr);
			if (attr){
				stype *st;
				cout << "col:" << *aiob->gob->get_int_ptr("targ_dist") << endl;
				Vector<string> v;
				aiob->gob->attr_names(v);
				cout << "attributes:" << endl;
				FORALL(v,it){
					st = aiob->gob->get_val_ptr(*it);
					switch(st->type){
						case stype::INT:
							PDEBUG( << *it << " :" << st->to_int() );
							break;
						case stype::CSTR:
							PDEBUG( << *it << " :" << st->to_cstr() );
							break;
						default:
							PDEBUG( << *it << " - unknown value type");
							
					}
				}
			}
			if (!aiob->hasState()){
				std::cout << endl<< "updating worker "<<aiob->getId() << std::endl;

				//  			Only one control center implemented
				
				//Path* path=HANDLER->graph->shortestPath(HANDLER->collector->getFirst<ControlCenter>(player_bitmap)->getLoc(), mineralsSet);
				Path * path = HANDLER->graph->shortestPath(aiob->getLoc(), mineralsSet);
				if (path==NULL || path->empty()) {
					cout << "path not found!" << endl;
					continue;
				}

				Mineral* mineral=NULL;

				FOR_EACH_OBJ(m,Mineral){
					if (m->getLoc()==path->back()) {
						mineral=m; break;
					}
				}
				assert(mineral);
				aiob->gather(mineral,HANDLER->collector->getFirst<ControlCenter>(player_bitmap));
				//if (mineral->workersNum>= mineralWorkersMax) mineralsSet.erase(mineral->getLoc());
			}
			aiob->setAction();
//			aiob->setMovement();
		}
		FOR_EACH_OBJ_OF(aiob, Worker, player_bitmap){
			aiob->setMovement();
		}
	}

	
}
