
#include <algorithm>

#include "Formation.H"
#include "Configuration.H"
#include "Agent.H"

using namespace std;

namespace bzAI{


	void Formation::addAgent(Agent* aiob){
		v.push_back(aiob);
	}

	void EnemyFollowY::addEnemy(Agent* enob){
		enemy.push_back(enob);
	}

	void EnemyFollowY::createFormation(Loc p, vec2 dir){
		vector<pair<sint4, Agent*> > hor_v, ene_v;
		FOREACH(it, v) hor_v.push_back(make_pair(it->getLoc().y,it));
		sort(ALL(hor_v));
		
		FOREACH(it, enemy) ene_v.push_back(make_pair(it->getLoc().y,it));
		sort(ALL(ene_v));

		vector<Agent*> cur_unit;

		static int group_dist = HANDLER->config->getInt("game4_group_dist");

		REP(i,hor_v.size()){
			cur_unit.push_back(hor_v[i].second);
			if (i==hor_v.size()-1 || hor_v[i+1].second->getLoc().y - group_dist > hor_v[i].second->getLoc().y){
				if (cur_unit.back()->getLoc().y > 768 - group_dist)
					REP(j,cur_unit.size()) cur_unit[j]->rotation_dir=dir.x>0 ? 1 : -1; else
				if (cur_unit.front()->getLoc().y < group_dist)
					REP(j,cur_unit.size()) cur_unit[j]->rotation_dir=dir.x > 0 ? -1 : 1; else

				REP(j,cur_unit.size())
					if (j<cur_unit.size()/2) cur_unit[j]->rotation_dir=dir.x>0 ? 1 : -1; else
						cur_unit[j]->rotation_dir=dir.x > 0 ? -1 : 1;
				cur_unit.clear();
			}
//			if (i<v.size()/2) hor_v[i].second->rotation_dir=1; else
//				hor_v[i].second->rotation_dir=-1;
			hor_v[i].second->gotoTarget(Loc(p.x,ene_v[i].second->getLoc().y));
		}
	}

	uint4 Formation::size(){
		return v.size();
	}

	LineFormation::LineFormation(sint4 d, line_rotation rtype):dist(d),rot_type(rtype){}

	void LineFormation::createFormation(Loc p, vec2 dir){
		vector<pair<sint4, Agent*> > hor_v;
		
		FOREACH(it, v) hor_v.push_back(make_pair(it->getLoc().y*dir.x-it->getLoc().x*dir.y,it));
		sort(ALL(hor_v));

		sint4 upper_rot, lower_rot;

		switch (rot_type){
			case ROTATE_RIGHT: upper_rot=lower_rot=-1;
					   break;
			case ROTATE_LEFT:  upper_rot=lower_rot=1;
					   break;
			case ROTATE_SIDES: upper_rot=1; lower_rot=-1;
					   break;
			case NO_ROTATE:    upper_rot=lower_rot=0;
		}

		REP(i, hor_v.size()){
			if (i<v.size()/2) 
				hor_v[i].second->rotation_dir=upper_rot; else 
				hor_v[i].second->rotation_dir=lower_rot;

			hor_v[i].second->gotoTarget(Loc(
						p.x-dir.y*dist*(i-(int)v.size()/2),
						p.y+dir.x*dist*(i-(int)v.size()/2)));

		}

	}

	WingsFormation::WingsFormation(sint4 dist, uint4 w_num, sint4 w_dist):
								form_middle(dist, LineFormation::ROTATE_SIDES), 
								form_right(dist, LineFormation::ROTATE_RIGHT), 
								form_left(dist, LineFormation::ROTATE_LEFT), 
								wing_num(w_num), wing_dist(w_dist), spread(0){}


	void WingsFormation::spreadLevel(real8 s){
		spread=s;
	}
	void WingsFormation::wingDistance(sint4 d){
		wing_dist = d;
	}

	void WingsFormation::createFormation(Loc p, vec2 dir){
		if (form_middle.size()==0 && form_right.size()==0 && form_left.size()==0){

			vector<pair<sint4, Agent*> > hor_v;

			FOREACH(it, v) hor_v.push_back(make_pair(it->getLoc().y*dir.x+it->getLoc().x*dir.y,it));
			sort(ALL(hor_v));

			REP(i,wing_num) form_left.addAgent(hor_v[i].second);
			FORL(i,wing_num, size()-1-wing_num) form_middle.addAgent(hor_v[i].second);
			FORL(i,size()-wing_num,size()-1) form_right.addAgent(hor_v[i].second);
		}

		form_middle.createFormation(p,dir);

		real8 d1 = form_middle.size()/2 * form_middle.dist + wing_dist;
		real8 d2 = form_left.size()/2 * form_left.dist;

		vec2 left_dir= (spread <= 0.5 ? vec2(1,0) : vec2(1,0).rotate( 1.0 * (spread - 0.5) * M_PI/2.0));
		vec2 right_dir= (spread <= 0.5 ? vec2(1,0) : vec2(1,0).rotate( -1.0 * (spread - 0.5) * M_PI/2.0));

		Loc left_pos(spread <= 0.5 ? 30 : (left_dir.y * d2),
			     -wing_dist - d2 - spread * d1);
		
		Loc right_pos(spread <= 0.5 ? 30 : (-right_dir.y * d2),
			     wing_dist + d2 + spread * d1);

		vec2 tmp = left_dir;
		left_dir.x = tmp.x * dir.x - tmp.y * dir.y;
		left_dir.y = tmp.x * dir.y + tmp.y * dir.x;

		tmp = right_dir;
		right_dir.x = tmp.x * dir.x - tmp.y * dir.y;
		right_dir.y = tmp.x * dir.y + tmp.y * dir.x;

		form_left.createFormation(
			Loc(p.x - left_pos.y * dir.y + left_pos.x * dir.x, p.y + left_pos.y * dir.x + left_pos.x * dir.y), left_dir);
		
		form_right.createFormation(
			Loc(p.x - right_pos.y * dir.y + right_pos.x * dir.x, p.y + right_pos.y * dir.x + right_pos.x * dir.y) ,right_dir);

	}

}
