
#include <cassert>
#include <set>
#include <queue>

#include "Game.H"
#include "GameObj.H"
#include "Global.H"
#include "Distance.H"

#include "Graph.H"
#include "Worker.H"
#include "Mineral.H"
#include "ControlCenter.H"
#include "AIState.H"
#include "AIEventHandler.H"

#include "Configuration.H"

#include "Path.H"
#include "AIGlobal.H"

using namespace std;

namespace bzAI {


int Graph::getVertex(Loc loc){
	if (coord2v.count(loc) == 0){		
		coord2v[loc] = num_v;
		v2coord[num_v] = loc;
		num_v++;
	}
	return coord2v[loc];
}
Loc Graph::getLoc(int v){
  return v2coord[v];
}

GameObj* Graph::getBoundary(line_seg a){
	*dummy->sod.x1 = a.l1.x;
	*dummy->sod.y1 = a.l1.y;
	*dummy->sod.x2 = a.l2.x;
	*dummy->sod.y2 = a.l2.y;
	*dummy->sod.shape = Object::LINE;
	return dummy;

}

bool Graph::testpath(line_seg a, sint4 d, AIObject* aiob){
	GameObj* dummy = getBoundary(a);
	if (coord2v.count(a.l2) && mineral_input[getVertex(a.l2)] &&coord2v.count(a.l1) &&  !mineral_input[getVertex(a.l1)]) d=1;
	if (coord2v.count(a.l2) && !mineral_input[getVertex(a.l2)] &&coord2v.count(a.l1) &&  mineral_input[getVertex(a.l1)]) d=1;
	if (coord2v.count(a.l1) && mineral_input[getVertex(a.l1)] && aiob!=NULL && a.l2 == aiob->getLoc()) d=1;
	FOR_EACH_OBJ(o, Obstacle){
		if ((*o->gob->sod.shape == Object::RECTANGLE || *o->gob->sod.shape == Object::LINE) && 
		    (min(*o->gob->sod.x1, *o->gob->sod.x2)> max(*dummy->sod.x1, *dummy->sod.x2)+d || 
		     max(*o->gob->sod.x2,*o->gob->sod.x1) < min(*dummy->sod.x1, *dummy->sod.x2)-d ||
		    (min(*o->gob->sod.y1, *o->gob->sod.y2)> max(*dummy->sod.y1, *dummy->sod.y2)+d || 
		     max(*o->gob->sod.y2,*o->gob->sod.y1) < min(*dummy->sod.y1, *dummy->sod.y2)-d))) continue;

		if (o!= aiob && Game::distance(*dummy, *o->gob)<d) return false;
	}
	
	return true;


/*	pun p1(a.l1.x, a.l1.y), p2(a.l2.x, a.l2.y);

	for (list<line_seg>::iterator t = bound.begin();t!= bound.end();++t){ 
		if (intersect(p1, p2, pun(t->l1.x, t->l1.y), 
					pun(t->l2.x, t->l2.y), d)) {return false;}
	}
	FORALL(res, it) if (a.l2 == it->first) { 
		FORALL(res, it2) if (it!= it2){
			d<? = (sint4)sqrt((double)distance2(it->first, it2->first))-6; 
		}
		break;
	}
	for (list<thick_point>::iterator t = res.begin();t!= res.end();++t){
		if (odl(p1, p2, pun(t->first.x, t->first.y))<d+t->second && a.l1!= t->first && a.l2!= t->first)
			return false;
	}
	return true;*/
}

struct p_vert {
	int v;
	double d;
	p_vert(int _v, double _d):v(_v), d(_d){}
};
inline bool operator < (const p_vert b, const p_vert a){
	return b.d>a.d;
}

void Graph::vert_vector::push_back(int v, double d){
	t[size]=v;
	w[size]=d;
	++size;
}
Graph::vert_vector::vert_vector():size(0){
}

Path* Graph::shortestPath(Loc p1, Loc p2, bool mirrored){
//	cout << "running shortest path "<<p1<<" "<<p2<<" :";
	int counter1=0,counter2=0;
        vert_vector* graph;
        if (mirrored) graph = mirror_g; 
        else graph = g;
	if (empty_graph){
		list<Loc> *r=new list<Loc>;
		r->push_back(p1);
		r->push_back(p2);
		return new Path(r,p1.distance(p2));
	}
	int v = closestFree(p1), w = closest(p2);
//	cout <<" from "<<getLoc(v)<<" to "<<getLoc(w)<<" ";
	static double d[MAX_VERT];
	static int pop[MAX_VERT];
	for (int i = 0;i<MAX_VERT;++i) pop[i] = -2;
	pop[v] = -1;
	for (int i = 0;i<MAX_VERT;++i) d[i] = 1000000000;
	d[v] = 0;
	priority_queue<p_vert> q;
	q.push(p_vert(v, 0));
	while (!q.empty()){
		int u = q.top().v;
		++counter2;
		if (q.top().d>d[u]+1e-7) { q.pop();continue;}
		q.pop();
		if (u == w) break;
		REP(i, graph[u].size){
                        int it = graph[u].t[i];
			double dist = graph[u].w[i];
			if (d[u] + dist<d[it] && !inactive[it]){
				d[it] = d[u]+dist;;
				q.push(p_vert(it, d[it]));
				pop[it] = u;
			}
                }
	}
	list<Loc> *r = new list<Loc>;
	for (int u = w;u!= -1;u = pop[u]) 
		if (u == -2) {
//			cout <<"not found, analized "<<counter2<<endl;
			return NULL;
		} else {
			r->push_front(Loc(v2coord[u].x, v2coord[u].y));
		}

	if (p1!= v2coord[v])r->push_front(Loc(p1.x, p1.y));
	if (p2!= v2coord[w])r->push_back(Loc(p2.x, p2.y));
//	cout <<"length "<<r->size()<<" analized "<<counter2<<endl;
	return new Path(r, d[w]);
}



// TODO jaka jest roznica w dzialaniu dwoch shortestPath? nie da sie ich
// ujednolicic? 
// A * nie dziala dla wielu targetow

Path* Graph::shortestPath(Loc p1, std::set<Loc>& endlocs, bool mirrored){
//	cout << "shortest path to endlocs visited: ";
	if (empty_graph){
		double dist=1000000000.0,tmp;
		Loc t;
		FOREACH(it,endlocs) if ((tmp=p1.distance(it))<dist){
			dist=tmp;
			t=it;
		}
		list<Loc> *r=new list<Loc>;
		r->push_back(p1);
		r->push_back(t);
		return new Path(r,sqrt((real8)distance2(p1,t)));
	}
        vert_vector* graph;
        if (mirrored) graph = mirror_g;
        else graph = g;
	set<sint4> ends;
	FORALL(endlocs, it) {
		//cout << "end pos " << *it<<endl;  
		if (coord2v.count(*it)) 
			ends.insert(coord2v[*it]);
	}

	int v = closestFree(p1);
	double d[MAX_VERT];
	int pop[MAX_VERT];
	int found = -1;
	for (int i = 0;i<MAX_VERT;++i) pop[i] = -2;
	pop[v] = -1;
	for (int i = 0;i<MAX_VERT;++i) d[i] = 1000000;
	d[v] = 0;
	priority_queue<p_vert> q;
	q.push(p_vert(v, 0));
	while (!q.empty()){
		int u = q.top().v;
	//	cout << u<<" ";
		if (q.top().d>d[u]) { q.pop();continue;}
		q.pop();
		if (ends.count(u)){ found = u; break;}
		REP(i, graph[u].size){
			int it = graph[u].t[i];
			double dist = graph[u].w[i];
			if (d[u]+dist<d[it] && !inactive[it]){
				d[it] = d[u]+dist;
				q.push(p_vert(it, d[it]));
				pop[it] = u;
			}
		}
	}
//	cout << endl;
	if (found == -1) return NULL;
	list<Loc> *r = new list<Loc>;
	int w = -2;
	double mind = 1000000;
	FORALL(ends, it){
		if (mind>d[(*it)]) {mind = d[(*it)];w = *it;}
	}

	for (int u = w;u!= -1;u = pop[u]) if (u == -2) { delete r; return NULL;} 
	else	r->push_front(v2coord[u]);
	if (p1!= v2coord[v])
		r->push_front(Loc(p1.x, p1.y));
	return new Path(r, d[found]);
}

Path* Graph::nearPath(Loc start_l, Loc end_l, double r, bool mirrored){
  int end_v = coord2v[end_l];
  std::set<Loc> targets;

  for (int v = 0; v < num_v; ++v){
    if (v_dist(end_v, v) < r){
      targets.insert(v2coord[v]);
    }

  }

  return shortestPath(start_l, targets, mirrored);
}



Graph* Graph::generate(Configuration* cfg){
	Graph* g=new Graph();
	g->empty_graph=false;

	FOR_EACH_OBJ(m, Mineral){
		Loc l = m->getLoc();
		Loc l1(l);
		l1.x -= l1.x%16;
		l1.y -= l1.y%16;

		
		Loc l2(l);
		l2.x -= l2.x%16-16;
		l2.y -= l2.y%16;
		
		Loc l3(l);
		l3.x -= l3.x%16-16;
		l3.y -= l3.y%16-16;
		
		Loc l4(l);
		l4.x -= l4.x%16;
		l4.y -= l4.y%16-16;


		g->mineral_input[g->getVertex(l1)]=true;
		g->mineral_input[g->getVertex(l2)]=true;
		g->mineral_input[g->getVertex(l3)]=true;
		g->mineral_input[g->getVertex(l4)]=true;

	}

	string graph_type = cfg->getString("type");
	cout <<"Generating graph, type= "<<graph_type<<endl;
	if (graph_type=="grid") 
		g->grid_layout(cfg->getInt("node_w"), 
			       cfg->getInt("node_h"), 
			       cfg->getInt("connect_dist"),
			       cfg->getInt("obstacle_dist")); else

	if (graph_type=="visibility") 
		g->visibility_layout(); else
	if (graph_type=="empty") g->empty_graph=true;

	return g;
}


Graph::Graph(){
	Game* game = &AIEventHandler::handler->state->gsm->get_game();
	dummy = game->get_cplayer_info().gof.create("boundary")->get_GameObj();
	assert(dummy);
//	cout << "Boundary name "<<changes->new_boundaries[0]->bp_name()<<endl;
	
	dummy->sod.x1 = new sintptr;
	dummy->sod.y1 = new sintptr;
	dummy->sod.x2 = new sintptr;
	dummy->sod.y2 = new sintptr;
	dummy->sod.shape = new sintptr;
	REP(i,MAX_VERT) inactive[i]=mineral_input[i]=false;
}

int gcd(int a, int b){
	if (b==0) return a;
	return gcd(b,a%b);
}
void Graph::grid_layout(int k, int l, int d1, int  d2){
	Game* game = &AIEventHandler::handler->state->gsm->get_game();
	const Map<GameTile> *map = &game->get_map();
	sint4 width = map->get_width()*game->get_tile_points(), 
	      height = map->get_height()*game->get_tile_points();
	sint4 odx = width/(k + 1), 
	      ody = height/(l + 1);
	int count = 0;
        ControlCenter* cc=NULL; 
	if (HANDLER->game_type==GAME_1) cc=HANDLER->collector->getFirst<ControlCenter>(player_bitmap); // jedno control center
        static int ccAccessRadius = HANDLER->config->getInt("graph_cc_access_radius");
        static int mineralAccessRadius = HANDLER->config->getInt("graph_mineral_access_radius"); 
	static int directed_edges = HANDLER->config->getInt("graph_directed_edges");
	for (sint4 x = odx; x < width; x += odx)
		for (sint4 y = ody; y < height; y+= ody){
			if (cc && distance2(cc->getLoc(), Loc(x, y))<ccAccessRadius && testpath(line_seg(Loc(x, y), cc->getLoc()), d2, cc) &&
					!(x >= *cc->gob->sod.x1 && x <= *cc->gob->sod.x2 && y >= *cc->gob->sod.y1 && y <= *cc->gob->sod.y2)){
				int v = getVertex(Loc(x, y)), w = getVertex(cc->getLoc());
				g[v].push_back(w,v_dist(v,w));
				mirror_g[w].push_back(v,v_dist(w,v));
			}
			FOR_EACH_OBJ(o, Mineral) {
				if (distance2(o->getLoc(), Loc(x, y))< mineralAccessRadius && testpath(line_seg(Loc(x, y), o->getLoc()), d2, o)){
					int v = getVertex(Loc(x, y)), w = getVertex(o->getLoc());
					g[v].push_back(w,v_dist(v,w));
					mirror_g[w].push_back(v, v_dist(w,v));
					//	g[w].push_back(v);
				}
			}
			for (sint4 x1 = x;x1 <= min(width-odx, x + d1);x1 += odx)
				for (sint4 y1 = max(y-d1, ody);y1 <= min(height - ody, y + d1);y1 += ody) if ((x1>x || y1>y) && abs(gcd((x-x1)/odx,(y-y1)/ody))==1 ) {
					if ((x1!= x || y1!= y)&& testpath(line_seg(Loc(x, y), Loc(x1, y1)), d2)){
						int v = getVertex(Loc(x, y)), w = getVertex(Loc(x1, y1));
						double dist=0;
						if (!directed_edges || (y1==y && (y/ody)%2==0)||(x1==x && (x/odx)%2==0)||(y1!=y && x1!=x && (x/odx+y/ody)%2==1))
							dist = v_dist(v,w); else
							dist = 10000.0 * v_dist(v,w);
						g[v].push_back(w,dist);
						mirror_g[w].push_back(v, dist);
						if (!directed_edges || (y1==y && (y/ody)%2==1)||(x1==x && (x/odx)%2==1)||(y1!=y && x1!=x && (x/odx+y/ody)%2==0))
							dist = v_dist(w,v); else
							dist = 10000.0 * v_dist(w,v);
						g[w].push_back(v,dist);
						mirror_g[v].push_back(w,dist);

						++count;
					}
				}
		}

        findConnectedComponents();
	cout << "grid graph: made "<< count <<" edges"<<endl;
}

void Graph::visibility_layout(){
	ERR("visibility graph not implemented");
}

void Graph::update(const GameChanges*){
//	ERR("Not implemented");
}

int Graph::closest(Loc p){
	sint4 d = 1000000, tmp;
	int v = 0; //BUG g[i].size > 1??????????
	for (int i = 0; i<num_v; ++i) if (!inactive[i] && (tmp = distance2(p, v2coord[i])) < d){ d = tmp; v = i;}
	return v;
}


//TODO za kazdym razem zbierane sa mineraly???
int Graph::closestFree(Loc p){
	sint4 d = 1000000, tmp;
	int v = 0;
	set<Loc> minerals;
	FOR_EACH_OBJ(o,Mineral) minerals.insert(o->getLoc());
        // po co g[i].size()>1?
	for (int i = 0; i < num_v; ++i)if (g[i].size>1 && connComps[i] == 1)
        {
		if (minerals.count(v2coord[i])) continue;
		if (!inactive[i] && (tmp = distance2(p, v2coord[i]))<d && testpath(line_seg(p, v2coord[i]), 1)){ d = tmp; v = i;}
	}
	return v;
}	



double Graph::v_dist(int u, int v){
	return sqrt((double)distance2(v2coord[u], v2coord[v]));
}

// zaznaczenie jednej spojnej skladowej zawierajacej controlCenter
void Graph::findConnectedComponents(){
	if (HANDLER->game_type!=GAME_1){
		REP(i,num_v) connComps[i] = 1;
		return;
	}
        stack<int> q;
        ControlCenter* cc = HANDLER->collector->getFirst<ControlCenter>(player_bitmap); // jedno control center, game1
        int it;
	set<Loc> minerals;
	FOR_EACH_OBJ(o,Mineral) minerals.insert(o->getLoc());

        if (cc){
          int v = coord2v[cc->getLoc()];
          for (int i = 0; i < num_v; ++i) connComps[i] = 0;
          q.push(v);
          while (!q.empty()){
            v = q.top();
            q.pop();
            connComps[v] = 1;
            /*for (int i = 0; i < g[v].size; ++i) {
              it = g[v].t[i];
              if (connComps[it]==0 && minerals.count(v2coord[it])==0){
                connComps[it] = 1;
                q.push(it);
              }
            }*/
            for (int i = 0; i < mirror_g[v].size; ++i){
              it = mirror_g[v].t[i];
              if (connComps[it]==0){
                connComps[it] = 1;
                q.push(it);
              }
            }
          }
        }

}

}
