#include <cmath>


#include "Global.H"
#include "GameObj.H"

#include "AIGlobal.H"
#include "Configuration.H"
#include "Worker.H"
#include "Graph.H"
#include "Path.H"
#include "Mineral.H"
#include "ControlCenter.H"
#include "Distance.H"


using namespace bzAI;
using namespace std;

Worker::Worker(GameObj* gob):
	Agent(gob),
	isWorking(false),
        mineral(NULL),
	fetched(0),
	turn(0),
	baseEncountered(0),
        inactivityTime(0),
	state(NONE),
	last_speed(0)
{
	real8 wmin = HANDLER->config->getReal8("worker_weight_min");
	real8 wmax = HANDLER->config->getReal8("worker_weight_max");
	weight = rrand() * (wmax - wmin) + wmin;
}




void Worker :: setState(State s){
	previousState = state;
	state = s;
  inactivityTime = 0;
	// PDEBUG("Worker.setState " << s);
}


bool Worker::reachedMineral(){
	return Distance::distance(*mineral->gob,*gob)<= mineral_dist;
}

bool Worker::reachedControlCenter(){
	return Distance::distance(*controlCenter->gob,*gob)<= mineral_dist;
}

void Worker::setAction(){
	dont_move = false;
	//cout << "worker "<<id<<": setAction"<<endl;
	switch(state){
		case MOVE_TO:
			if (reachedMineral()){
				stop();
				setState( MINE );
			} else {
			}
			break;	
		case MOVE_BACK:
			if (reachedControlCenter()){
				stop();
				setState(UNLOAD);
			} else {
			}

			break;
				
		case MINE:
			/* setState(MOVE_BACK);
			gotoTarget(controlCenter);
			break; // mining turned off */
			if (gob->get_int("minerals") >=10){	//mining ended
				//obj->component("pickaxe")->set_action("stop",Vector<sint4>());
				//stop();
				setState(MOVE_BACK);
				HANDLER->graph->inactive[HANDLER->graph->getVertex(last_access_loc)]=false;
				gotoTarget(controlCenter);
				isWorking=false;
				
				--mineral->workersNum;

				mineral=NULL;

			} else if (!isWorking) {
                                cout << "start mining" << endl;
				mine();
                                dont_move = true;
				isWorking=true;

			}else{
                                dont_move = true;
				/*DENTER();
				PDEBUG("pickaxe is active: " << gob->component("pickaxe")->get_int("active"));
				PDEBUG("mineral name:" << mineral->gob->bp_name());
				PDEBUG("mineral.num_miners:" << mineral->gob->get_int_ptr("num_miners"));
				PDEBUG("worker.minerals:" <<gob->get_int("minerals"));
				DEXIT();*/
			}
			
			break;
		case UNLOAD:
			if (gob->get_int("minerals") == 0){	//unloading ended
				setState(NONE);
/*				setState(MOVE_TO);
				gotoTarget(mineral);
				isWorking=false;
                                dont_move = true; */
//				static PlayerInfo& my_info=AIEventHandler::.get_game().get_cplayer_info();
//				cout << "====SCORE : " << my_info.global_obj("player")->get_int("minerals")<<endl;
			}
			else if (!isWorking){
				unload();
                                dont_move = true;
				isWorking=true;
			}		
			break;
		
		default:/*impossible*/
			cout <<"impossible state"<<endl;exit(-1);
	}
}
bool Worker::hasState(){
	return state!=NONE;
}
void Worker::gather(Mineral* m, ControlCenter* c){
	DENTER();
        
        if (mineral)
          --mineral->workersNum;

	mineral=m;
        ++mineral->workersNum;
	controlCenter=c;
	gotoTarget(mineral);
	HANDLER->graph->inactive[HANDLER->graph->getVertex(last_access_loc=path->beforeLast())]=true;
	PDEBUG("Worker " << getId() << ": gather mineral " << m->getId() << " at" << m->getLoc());
	setState(MOVE_TO);
	isWorking = false;

	DEXIT();
	
//	setPath(Loc(*res->sod.x, *res->sod.y)); 
}
void Worker::mine(){
	int nr;
	//DENTER();
	Vector<sint4> params;
	assert(mineral);
	params.push_back(mineral->getId());
	//PDEBUG("mineral_id :"<< mineral->getId());
	if (!(nr = gob->component("pickaxe")->set_action("mine", params)))
		PDEBUG("mine failed!");
	//PDEBUG("mine action nr: " << nr);
	//DEXIT();
}
void Worker::unload(){
        //DENTER();
	Vector<sint4> params;
	params.push_back(controlCenter->getId());
	gob->set_action("return_resources", params);
        //DEXIT();
}

void Worker::updateInactivity(){
	static int maxInactivityTime = HANDLER->config->getInt("worker_max_inactivity_time");
	if ((!path || path->empty()) && (state == MOVE_TO || state == MOVE_BACK)){
		++inactivityTime;
		cout << "inactivity detected!:" << inactivityTime<< endl;
	}
	if (inactivityTime >= maxInactivityTime){
		inactivityTime = 0;
		if (state == MOVE_TO)
			gotoTarget(mineral);
		else 
			gotoTarget(controlCenter);
	}
}
void Worker::updateMineral(){
	if (mineral && mineral->isDead){
		HANDLER->graph->inactive[HANDLER->graph->getVertex(last_access_loc)]=false;

		if (state == MOVE_TO)
			setState(NONE);
		else if (state == MINE)
			setState(NONE);
		isWorking = false;

	}

}

