// $Id: aiclient1_main.C,v 1.5 2007/05/21 15:27:43 mbrzozowski Exp $

// Co za bzdura z Options??? 4 rozne sposoby dodawania opcji do gry :/
#include "GfxGlobal.H"
#include "Options.H"
#include "GfxModule.H"
#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"

#include "AIEventHandler.H"
#include "AIState.H"
#include "AIGUI.H"
static char *VERSION = "aiclient v0.2";
int DINDENT = 0;

using namespace std;
using namespace bzAI;

// loop is called
static const sint4 GFX_X = 900;
static const sint4 GFX_Y = 740;
class Looper : public LoopFunctor
{
	public :

		Looper(AIState *state_) : state(state_) {}

		void loop() const {

			if (state->gfxm) {

				if (state->gfxm->get_quit()) { state->quit = true; exit(0); } // should return

				bool refresh;
				Options::get("-refresh", refresh);

				if (refresh) {

					// maximal gfx refresh rate

					if (!state->just_drew) 
						state->gfxm->draw();
					else
						state->just_drew = false;
				}
			}

			// looks for server messages
			// if one or more arrived, calls handlers
			// if not, sleeps for one millisecond
			if (!state->gsm->recv_view()) SDL_Delay(1);
		}

	private:
		AIState *state;
};


int main(int argc, char **argv)
{


	signals_shut_down(true);

	Options opt("AIClient");
	opt.put("-gfxtype", 0, "display world in: 2 - 2d, 3 - 3d, else - none");
	opt.put("-game_type",1, "tournament game to play");
	opt.put("-pathfind_test", "run a pathfinding test");
	opt.put("-cinfo", "display game state changes");
	opt.put("-tinfo", "display tile info");
	opt.put("-oinfo", "display object info");
	opt.put("-minfo", "display map info");    

	opt.put("-attr", "wypisz atrybuty jednostki");    


	// populate opt with options from various modules

	GameStateModule::Options::add();
	GfxModule::Options::add();
	Game::Options::add();
	AIGUI::add_options();

	// process command line arguments
	if (Options::process(argc, argv, cerr, VERSION)) exit(20);

	// initialize SDL
	SDLinit::network_init();

	// populate option objects with command line values
	// and create modules

	GameStateModule::Options gsmo;
	GameStateModule gsm(gsmo);

	GfxModule::Options gfxmo;
	GfxModule *gfxm = 0;
	AIGUI *gui = 0;
	// change seed value to time(0) if 0

	sint4 seed;
	Options::get("-seed", seed);

	if (!seed) seed = time(0);

	Options::set("-seed", seed);
	cout << "seed=" << seed << endl;

	sint4 gfx_type = 0;
	Options::get("-gfxtype", gfx_type);

	switch(gfx_type) {
		case 2:
			SDLinit::video_init();
			gui = new AIGUI;
			break;
		case 3:
			SDLinit::video_init();
			gfxm = new GfxModule;
			break;
		default:
			break;
	}
	// gives event handler access to the state
	AIState state(&gsm, gfxm, gui);

	AIEventHandler sev(&state, seed);

	// gsm events are handled in sev
	gsm.add_handler(&sev);

	// connect game state module to server
	if (!gsm.connect()) 
		ERR("connection problems");

	Looper l(&state);

	if (gfxm) {

		// gfx mode
		glutInit(&argc, argv);
		gfxm->init(gsm.get_game(), gsm.get_changes(), gsm.get_action_changes(), gfxmo);

		// game loop (could become a thread)
		gfxm->start_loop(&l);
		delete gfxm;
	} if (gui){
		gui->init(GFX_X, GFX_Y, gsm.get_game());
		while (!state.quit) l.loop();
		delete gui;
	}else {
		while (!state.quit) l.loop();
	}

	return 0;
}
