/* Game Class Implementation (with trivial test code)
 * Krysta Mirzayans  10/05/06
 */

#include "game.h"

//BASIC TESTING OF GAME CLASS FUNCTIONALITY:
int main()
{
  cout << "TEST CTOR: game 1" << endl;
  int array[] = {1, 2};
  Game test1 = Game(array);
  test1.display();  
  
  cout << "TEST CCTOR: game 2" << endl;
  Game test2 = test1;
  test2.display();

  cout << "TEST CTOR: game 3" << endl;
  int array2[] = {3, 4};
  Game test3 = Game(array2);
  test3.display();

  cout << "TEST ASSIGN OP: game 2 = game 3" << endl;
  test2 = test3;
  test2.display();

  cout << "PLAY game3" << endl;
  test3.play();
  test3.display();

  //destructor checked with valgrind: no leaks possible

  return 0;
}


 
//Game class member function implementation:

/* Game constructor:
 * 
 * Initializes the game players and sets the result to -1
 * (flag that the result is not yet valid).
 * The number of expected players depends on the static variable type.
 */
Game::Game(int *opponents)
{
  int numPlayers = Game::getNumPlayers();
  
  result = new int[numPlayers];
  valid = 0; //have not yet played - invalid results
  
  players = new int[numPlayers];
  for(int i=0; i < numPlayers; ++i)
    {
      players[i] = opponents[i];
      result[i] = -1;
    }
}

/* Game default destructor:
 *
 * Deletes memory for the Game object
 */
Game::~Game()
{
  delete[] players;
}

/* Game copy constructor:
 *
 * Copies Game paramater's values into this Game
 */
Game::Game(const Game& source)
{
  valid = source.valid;
  
  int numPlayers = Game::getNumPlayers();
  
  players = new int[numPlayers];
  result = new int[numPlayers];  
  
  for(int i=0; i<numPlayers; ++i)
    {
      players[i] = source.players[i];
      result[i] = source.result[i];
    }
}

/* Game deep-copy assignment operator:
 *
 */
Game &Game::operator= (const Game& source)
{
  if(&source != this)
    {
      valid = source.valid;
      
      delete[] result;
      delete[] players;
      int numPlayers = Game::getNumPlayers();
      players = new int[numPlayers];
      result = new int[numPlayers];

      for(int i=0; i<numPlayers; ++i)
	{
	  players[i] = source.players[i];
	  result[i] = source.result[i];
	}
    }
  return *this;
}
  
/* Pre: Game::type is the static variable containing the type
 * Post: Returns the number of players involved in a game of this type
 */
  int Game::getNumPlayers()
{
  int numPlayers = 0;
  switch(Game::type)
    {
    case TYPE1:
      numPlayers = 1;
      break;
    case TYPE2:
      numPlayers = 2;
      break;
    case TYPE3:
      numPlayers = 2;
      break;
    default:                             //what's it supposed to do?
      cerr << "Not a valid game type!\n";
      exit(10);
    }
  
  return numPlayers;
}

/* Public read access:
 * Post: Returns the players array
 */
int *Game::getPlayers()
{
  return players;
}

/* Displays the Game object
 */
void Game::display()
{
  cout << "******************************\n";
  cout << "Players: ";
  int numPlayers = Game::getNumPlayers();
  for(int i=0; i<numPlayers; ++i)
    cout << players[i] << " ";

  cout << "\nResult: ";
  for(int i=0; i<numPlayers; ++i)
    cout << result[i] << " ";

  cout << "\nValid Results? (0-invalid): " << valid << endl;
  cout << "******************************\n\n";
  return;
}

/* Plays this game: will have to actually do something later!
 */
void Game::play()
{
  int numPlayers = Game::getNumPlayers();
  for(int i=0; i<numPlayers; ++i)
    result[i] = 1;
  valid = 1;
}
