/* Krysta Mirzayans 1083757
 * 
 * Assignment 7 Question 3
 * Contains function popen2
 */

#include <unistd.h>
#include <sys/types.h>
#include <iostream>

using namespace std;

//FUNCTION DECLARATION:
pid_t popen2(const char *shell_cmd, int *p_fd_in, int *p_fd_out);

int main()
{
  //Test Case 1: Sort Example
  cout << "\n\nsort example:\n";
  int fd_in, fd_out;
  pid_t pid = popen2("sort | uniq", &fd_in, &fd_out);
  
  if(pid <= 0)
    {
      cerr << "Unable to exec prog";
    }
  
  write(fd_in, "A\n", 2);
  write(fd_in, "B\n", 2);
  write(fd_in, "C\n", 2);
  write(fd_in, "A\n", 2);
  close(fd_in);  //end of input

  while(1)  //read process output
    {
      char c;
      if(read(fd_out, &c, 1) <= 0)
	break; //no data
      cout << c;
    }

  //Test Case 2: ls example
  cout << "\n\nls example:\n";
  int fd_in1 =0;
  int fd_out1;
  pid_t pid1 = popen2("ls", &fd_in1, &fd_out1);
  
  if(pid1 <= 0)
    {
      cerr << "Unable to exec prog";
    }
  
  while(1)  //read process output
    {
      char c1;
      if(read(fd_out1, &c1, 1) <= 0)
	break; //no data
      cout << c1; 
    }

  //Test Case 3: an invalid shell command
  cout << "\n\ninvalid shell command example:\n";
  int fd_in3 =0;
  int fd_out3;
  pid_t pid3 = popen2("dksjfhahf", &fd_in3, &fd_out3);
  
  if(pid3 <= 0)
    {
      cerr << "Unable to exec prog";
    }
  
  while(1)  //read process output
    {
      char c3;
      if(read(fd_out3, &c3, 1) <= 0)
	break; //no data
      cout << c3; 
    }


  return 0;
}

//FUNCTION DEFINITION:

/* Pre: The shell command that should be executed is passed as a parameter.
 *      Values of 0 for p_fd_in or p_fd_out indicates shell command doesn't
 *      use stin or stdout respectively, so they should be closed.
 * Post: Makes accessible both the standard input and output of the shell 
 *       process it creates.
 *       Two pipes are created: one provides standard input for the shell 
 *       commmand, and the other for passing back its standard output.
 *       The pipe file descriptors for the caller to use are pointed to by 
 *       p_fd_in and p_fd_out.
 */      
pid_t popen2(const char *shell_cmd, int *p_fd_in, int *p_fd_out)
{
  //CREATING TWO PIPES:
  int fds_processInput[2];  //pipe for process input
  int fds_processOutput[2]; //pipe for process output
  
  if(pipe(fds_processInput) != 0) //create process input pipe
    {
      cerr << "pipe (process input) failed\n";
      exit(1);
    }
  
  if(pipe(fds_processOutput) != 0) //create process output pipe
    {
      cerr << "pipe (process output) failed\n";
      exit(1);
    }
  
  //FORKING A CHILD PROCESS:
  pid_t pid;
  if((pid = fork()) < 0)
    {
      cerr << "fork failed\n";
      exit(2);
    }
  
  //CONNECT THE CORRECT PIPE ENDS IN THE CHILD:
  if(pid == 0)  //child process
    {
      //for process input pipe:
      close(fds_processInput[1]);   //close output
      dup2(fds_processInput[0], 0); //close fd 0, fd 0 = fds_processInput[0]
      
      //for process output pipe:
      close(fds_processOutput[0]);   //close input
      dup2(fds_processOutput[1], 1); //close fd 1, fd 1 = fds_processOutput[1]
      

      execl("/bin/sh", "sh", "-c", shell_cmd, 0 ); 
      cerr << "failed to run shell_cmd\n";
    }
  else  //parent process
    {
      //for process input pipe:
      close(fds_processInput[0]);   //close input
      
      //for process output pipe:
      close(fds_processOutput[1]);   //close output

      if(p_fd_in == 0)
	close(fds_processInput[1]);
      else
	*p_fd_in = fds_processInput[1];
      
      if(p_fd_out == 0)
	close(fds_processOutput[0]);
      else
	*p_fd_out = fds_processOutput[0];

    }
  return pid; 
}
