#ifndef Profiler_H
#define Profiler_H

/** @file Profiler.H
    Hierarchical profiler.

    $Id: Profiler.H,v 1.7 2006/05/10 08:24:28 ddeutscher Exp $
    $Source: /usr/bodo1/cvs/cvs/orts3/libs/kernel/src/Profiler.H,v $
*/

// This is an ORTS file (c) Tim Furtak, licensed under the GPL

#include "Global.H"
#include "CycleCounter.H"

#define PROFILE(n) Profiler _prof_##__LINE__(n);

//===================================================================

class ProfileLevel {

private:
  void tick();

public:
  static const sint4 window = 5;

  std::string my_name;
  real8 total_time;
  real8 avg_time;         ///< Time averaged over 'window' steps.
  real8 win_time[window]; ///< Buckets for storing the last few times.
  sint4 window_ind;       ///< Index of the current bucket.
  std::map<std::string, ProfileLevel*> children;
  ProfileLevel *up;

  ProfileLevel(const std::string &n = "ROOT");

  friend class Profiler;
};

//===================================================================

/** Measures the amount of time that each Profiler object exists
    and stores the results hierarchically.
    Usage: enclose the area of concern within scope braces and
           use the PROFILE macro to label the region.
*/

class Profiler {

private:

  static ProfileLevel root;
  static ProfileLevel *cur;
  CycleCounter cc;
  ProfileLevel *pl;

public:

  Profiler(const std::string &name);
  ~Profiler();
  static ProfileLevel* get_root() { return &root; }
  static void tick();
};

//===================================================================

// typedef is needed since the comma in the map definition breaks the macro expansion
typedef std::map<std::string,ProfileLevel*> Name2ProfileLevelMap;
REGISTER_TYPEOF(5001, Name2ProfileLevelMap::iterator);

#if 0
#ifndef GCC

// fixme: what is this? this must be replaced by calling a Typeof.H macro
template<> struct typeof_class<1300> { typedef WrapType< std::map<std::string,ProfileLevel*>::iterator >::U V; };
char (*typeof_fct(const WrapType< std::map<std::string,ProfileLevel*>::iterator >::U &))[1300];
#endif
#endif

#endif
