/*
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/
*/

#ifndef OgreGfxModule_H_
#define OgreGfxModule_H_

#include "Options.H"
#include "ShutDown.H"
#include "Game.H"
#include "GameChanges.H"
#include "LoopFunctor.H"
#include "SDLinit.H"
#include "ObjectManager.H"

#include "TurretAI.H"
#include "GatherAI.H"

#include "ClientState.H"
#include "TerrainModule.H"

#include "GfxModule.H"
#include "SoundMixer.H"

#define OGRE_RELEASE_ASSERT 0

#include <Ogre.h>
#include <OgreKeyEvent.h>
#include <OgreEventListeners.h>
#include <OgreStringConverter.h>
#include <OgreException.h>
#include "OgreGfxModuleFrameListener.H"

class OgreGfxModuleFrameListener;

class OgreGfxModule{

public:
  /**
     Builds and initializes the Ogre core and all singleton's needed by the ogre engine.
     NOTE: because of the Ogre Singleton rame-work you should not create more than one 
     instance of this class and any time.
   */
  OgreGfxModule();

  /**
     In order for this OgreGfxModule to process changes correctly, it needs a reference to
     the game state module from orts
     \param a reference to the gsm
   */
  bool init(GameStateModule*);

  /**
     As the name suggests, this method has Ogre render exactly one frame. This method should be
     called again and again in a loop of some kind to keep rendering.
     \return if false is returned, then the module is indicating that it's time to quit
   */
  bool renderOneFrame();

  /**
     this should be one of the first things called by GfxModule::process_changes()
   */
  void process_changes();

  /**
     deallocates all allocated resources
   */
  ~OgreGfxModule();

  /**
     a reference to the ogre graphic module's event-handler
   */
  OgreGfxModuleFrameListener* mFrameListener;

private:
  //note: look in the .C file to view descriptions of private members

  void init_setupResources();
  bool init_configure();
  void init_createCamera();
  void init_createViewports();
  void init_createFrameListener();
  void createScene();

  Ogre::SceneManager* mSceneMgr;
  Ogre::InputReader* mInputDevice;
  Ogre::Root* mRoot;
  Ogre::RenderWindow* mWindow;
  Ogre::Camera* mCamera;

  GameStateModule* gsm;
};

#endif
