#ifndef GLOBALPATHFINDER_H
#define GLOBALPATHFINDER_H

#include "Object.H"
#include "TerrainBase.H"

class GlobalPathfinder{
public:

  typedef TerrainBase::Loc Loc;
  typedef TerrainBase::Path Path;
  typedef TerrainBase::Segment Segment;
  typedef TerrainBase::ConsiderObjects ConsiderObjects;
  
  GlobalPathfinder() {};

  virtual ~GlobalPathfinder(){};
  
  virtual void insert_object(const Object *obj) =0;
  virtual void remove_object(const Object *obj)= 0;
  virtual void update_object(const Object *obj) =0;
  virtual void insert_boundaries(const Vector<Segment> &s) =0;
  
   /** Perform pathfinding to a location, return true iff path found, and the path itself in 'output'.
        If found then output is guaranteed to include at least one waypoint for the goal. 
        Waypoints are listed in reverse order, goal is in locs[0]. */
  virtual bool find_path(const Object *obj, const Loc &goal, Path& output,  ConsiderObjects consider) = 0;
    /** Perform pathfinding to a target object, return true iff path found, and the path itself in 'output'
        If found then output is guaranteed to include at least one waypoint for the goal. 
        Waypoints are listed in reverse order, goal is in locs[0]. */
  virtual bool find_path(const Object *obj, const Object *goal, Path& output, ConsiderObjects consider) = 0;

};

#endif
