#ifndef PathfinderBasicImp_H
#define PathfinderBasicImp_H

// $Id: PathfinderBasicImp.H,v 1.4 2006/06/30 20:21:56 nicholas Exp $
// This is an ORTS file (c) Michael Buro, Nick Wiebe, licensed under the GPL


#include "TerrainBase.H"
#include "PathExecutor.H"
#include "PathfindTask.H"
#include "TerrainBasicImp.H"

/// \brief Implementation of the separation of Execution from Planning.
/** 
PathfinderBasicImp is be used to separate Pathfinding from Path Execution
using the TerrainBase interface. It uses PathfindTask, a wrapper for
TerrainBase::Task, to associate a task with a path so that the bookkeeping for
the Path Execution system and Pathfinding system are shared. The bookkeeping is
done by TerrainBasicImp

From the TerrainBase interface, the PathfinderBasicImp implements: 
execute_tasks - calls the PathExecutor to generate the necessary move commands and
status messages.
add_obj, remove_obj, update_obj, add_segments - passes the information on to the PathExecutor

The inheritance is getting a bit absurd at this point, though...
*/
class PathfinderBasicImp : public TerrainBasicImp<PathfindTask>
{
public:

  PathfinderBasicImp(PathExecutor &pe_) : TerrainBasicImp<PathfindTask>(), pe(pe_){};
  PathfinderBasicImp(const PathfinderBasicImp &other) 
    : TerrainBasicImp<PathfindTask>(other), pe(other.pe) {};
  virtual ~PathfinderBasicImp() {};

  /**
    Executes tasks by calling the PathExecutor.
  */
  virtual void execute_tasks(Vector<MoveCmd> &cmds, Vector<StatusMsg> &msgs);

/** \brief View Messages
  In derived classes, their version of these methods should call the PathfinderBasicImp::X()
  method first, so that the path executor gets the view updates.
*/
  virtual void add_obj(const Object *obj);
  virtual void remove_obj(const Object *obj);
  virtual void update_obj(const Object *obj);
  virtual void add_segments(const Vector<Segment> &segs);

protected:

  PathExecutor      &pe;

};
#endif
