#ifndef SI_SIMULATION_AI_H
#define SI_SIMULATION_AI_H


#include "Global.H"

#include "GameObj.H"
#include "LowAI.H"
#include "EventHandler.H"
#include "ClientModule.H"
#include "TerrainBase.H"

#include "SI_Strategies.h"

class GameStateModule;

namespace Simul{
//===================================================================

  class Task;
  class SimulationState;

  class ExtraObjectData {
  public:

    ExtraObjectData(){
      target = NULL;
      location = TerrainBase::Loc(-1,-1);
      state = NONE;
      isDone = false;
      hasReplanned = false;
    }

    GameObj *target;
    TerrainBase::Loc location;
    enum State {NONE,MOVE,DEFEND,ATTACK};
    State state;
    bool isDone;
    bool hasReplanned;
  };

//-------------------------------------------------------------------

/**
 * This AI module causes all units to attack the nearest (geometric distance)
 * base.
 */
class SimulationAI : public LowAI<ExtraObjectData>, public EventHandler, public ClientModule{
protected:
  //std::set<GameObj*> bases; ///< Enemy Bases

  GameStateModule &gsm;
  SimulationState* simulationState;


  //Vector<GameObj*> need_act; ///< GameObjects needing an action in the next tick
  /**
   * Removes the given GameObj from the control of this class.
   */
  //void stop(GameObj *obj);


  /**
   * Finds the nearest base to travel to, based on geometric distance.
   */
  //GameObj* find_target(GameObj *targeter, GameObj *baseToAvoid = 0);
public:

  static const sint4 FROM;
  static void add_options(void);
  SimulationAI(GameStateModule &gsm);
  ~SimulationAI();

  /**
   * Causes the units in the need_act vector to act according to
   * their current state.
   */
  void act(GameStateModule &gsm);

  //static const std::string bp_name; ///< ?

  void add(GameObj*) { ERR("?"); }

  /**
   * Addes an enemy base to the list of enemy bases, and updates targets
   */
  //void add_base(GameObj *gob);

  void del(GameObj*) { ERR("?"); }

  /**
   * Deletes a base from the list of enemy bases, and updates targets
   */
  //void del_base(GameObj *gob);

  /**
   * Handles events sent to SuicideAttackAI from other modules
   */
  bool handle_event(const Event &e);

  bool informObjectsOfDeath(GameObj* obj);
  bool isControllable(GameObj* obj);

  sint4 doPath(TerrainBase::Task& task);
  sint4 doCombat(Vector<GameObj::WPtr> units, GameObj::WPtr target);
  sint4 stopCombat(Vector<GameObj::WPtr> units);

  GameStateModule& get_gsm(){ return gsm;}

};

}

#endif

