/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Server.H"
#include "ServerClient.H"
#include "System.H"

Server::~Server() {
  FORALL (CLIENT_CONT, clients, it) delete *it;
}

sint4 Server::init_listen(int port)
{
  sint4 ok, i;
  sint4 my_socket;
     
  FOREVER {

    my_socket = System::sock_open(PF_INET, SOCK_STREAM, 0);
    if (my_socket < 0) {
      REM("port not available");
      sleep(5);
      continue;
    } 
      
    i = 1;
    ok = System::setsockopt(my_socket, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(int));
    if (ok < 0) ERR("setsockopt failed");

    if (System::sock_bind(my_socket, AF_INET, port) != 0) {
      REM("socket bind failed");
      continue;
    }

    if (System::sock_listen(my_socket, 4) != 0) {
      REM("socket listen failed");
      continue;
    }
    
    break;
  }

  socket = my_socket;
  return my_socket;
}


static sint4 my_set_sock_opt(int s, int cmd, int opt, bool report=true)
{
  int ok = setsockopt(s, SOL_SOCKET, cmd, &opt, sizeof(opt));
  if (report && ok < 0) ERR("set_socket_opt failed"); 
  return ok;
}


static sint4 my_set_fl(int f, int flags, bool report=true)
{ 
  int val = System::fcntl_arg(f, F_GETFL, 0);
  val |= flags;
  sint4 ok = System::fcntl_arg(f, F_SETFL, val);
  if (report && ok < 0) ERR("my_set_fcntl failed");
  return ok;
}

 
void Server::accept_connection()
{
  sint4 s;

  if ((s = System::sock_accept(socket, false)) < 0) {
    REM("!!! accept error");
    return;
  }
  
  ServerClient *p_client = new ServerClient(ibl, obl);

  if (p_client == 0) {
    REM("!!! no free slot");
    close(s); return;
  }

  my_set_fl(s, O_NONBLOCK);
  my_set_fl(s, O_NDELAY);
  my_set_sock_opt(s, SO_DEBUG,     0);
  my_set_sock_opt(s, SO_REUSEADDR, 1);
  my_set_sock_opt(s, SO_KEEPALIVE, 1);
  my_set_sock_opt(s, SO_DONTROUTE, 0);
  // my_set_sock_opt(s, SO_LINGER,    0);
  my_set_sock_opt(s, SO_BROADCAST, 0);
  my_set_sock_opt(s, SO_OOBINLINE, 1);

  /*
    my_set_sock_opt( s, SO_USELOOPBACK, 1 ); LINUX doesn't support this !!!
    my_set_sock_opt( s, SO_SNDBUF, 256 );
    my_set_sock_opt( s, SO_RCVBUF, 256 );
  */

  p_client->init();
  p_client->sb.set_socket(s);
  clients.push_back(p_client);
    
  // sprintf(p_client->host, "%s", (he ? he->h_name : dotted));

  cout << "client connected" << endl;
}


bool Server::handle_connection()
{
  fd_set         readfds;
  int            i;
  struct timeval timeout;
    
  timeout.tv_sec  = 0;
  timeout.tv_usec = 0;
  FD_ZERO(&readfds);
  FD_SET(socket, &readfds);

  i = System::select_desc(socket+1, readfds, *(fd_set*)0, *(fd_set*)0, &timeout);

  if (i < 0 && errno != EINTR) ERR("select connect");

  if (i < 0) REM("connect select error");
  if (i > 0) {
    accept_connection();
    return true;
  }

  return false;
}

