/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Global.H"
#include <vector>
#include "RTS_Game.H"
#include <SDL.h>
#include "SDL_gfxPrimitives.h"
#include <signal.h>
#include "CycleCounter.H"
#include "RTS_Options.H"
#include "RT_Server.H"
#include "RT_ClientEnv.H"
#include <time.h>

void signal_handler(int) { exit(1); }

const char *ORTS_VERSION = "ORTS version 1.00 (2002-03-07)";

extern sint4 EF_PROTECT_FREE, EF_FILL, EF_PROTECT_BELOW;

RTS_Options opt;


int main(sint4 argc, char **argv) {

#if 0
  // aggressive efence checks
  EF_PROTECT_FREE = 1;
  EF_FILL = 0xee;
  EF_PROTECT_BELOW = 1;
#endif

#if 0
  // frame info file test

  TickInfo ti;

  FILE *fp = fopen("history1", "r");
  if (!fp) ERR("foo");
  FILE *fpo = fopen("aaa", "w");

  FOREVER {
    ti.read(fp);
    if (feof(fp)) ERR("OK"); // file history1 and aaa should be equal
    if (ferror(fp)) ERR("read error");
    ti.write(fpo);
  }

  exit(0);
#endif
  
  ostringstream oserr;
  
  if (opt.process(argc, argv, cerr, ORTS_VERSION)) exit(20);

  sint4 seed;
  opt.get("-seed", seed);

  if (seed == 0) seed = time(0);
  cout << "seed=" << seed << endl;
  srand(seed);
  srand48(seed);  // used by STL

  bool display;
  opt.get("-disp", display);

  if (display) {
  
    // Initialize the SDL library
    
    if (SDL_Init(SDL_INIT_VIDEO |
		 SDL_HWSURFACE |
		 SDL_DOUBLEBUF |    
		 //	       SDL_SRCALPHA |
		 SDL_INIT_NOPARACHUTE) < 0) {
      fprintf(stderr,
	      "Couldn't initialize SDL: %s\n", SDL_GetError());
      exit(1);
    }

    atexit(SDL_Quit);
  }

  signal(SIGINT,  signal_handler);
  signal(SIGTERM, signal_handler);  
  signal(SIGKILL, signal_handler);
  
  string mode;
  opt.get("-m", mode);

  if (mode == "server" || mode == "ggs") {

    server_mode(opt);
    exit(0);

  } else if (mode == "client") {

    RT_ClientEnv ce(opt);
    ce.run();
    exit(0);
  }

  if (mode != "game") ERR("unknown mode");
  
  RTS_Game game(opt);
  game.simulation();
  cout << "game finished" << endl;
  return 0;
}
