// $Id: AI_Common.H,v 1.1.1.1 2004/10/11 09:09:20 orts_mburo Exp $

// This is an ORTS file (c) Michael Chung, licensed under the GPL

#ifndef AI_Common_H
#define AI_Common_H

#include "Coord.H"

class Game;
class GameObj;

//----------------------------------------------------------------------
// Game-specific Definitions
//----------------------------------------------------------------------

// obj_type values (must match blueprints)
#define TYPE_UNDEF  0  // abstract object
#define TYPE_UNIT   1  // objects that can move and perform actions
#define TYPE_POST   2  // a landmark indicating the key flag location
#define TYPE_FLAG   3  // a flag object
#define TYPE_GLOBAL 4  // a global object (e.g. player)


#define MAX_DIST (100.0)

// hard-coded range values
// scripted max range is 80, but we might like to get closer to the
// target (to avoid letting it get away)
// note: also, there may be a slight error from non-zero object size
#define ATTACK_RANGE (80)

// range in tiles
#define ATTACK_RANGE_TILES (ATTACK_RANGE / AREA_SIZE)

#define MAX_HP (100)

// assume fixed damage in AI for now
#define ATTACK_DAMAGE (10)

// for now, area size = tile size
#define AREA_SIZE (16)

// hard coded for now - known post locations
#define POST_1_LOC  Point(13, 3)
#define POST_2_LOC  Point(13, 24)

#define node_id_type int


//----------------------------------------------------------------------
// Debug macros
//----------------------------------------------------------------------

#define DB(X) \
  cout << #X << " = " << X << endl;

#define FORMAT_PT(PT) \
  "[" << PT.x << "," << PT.y << "]"
  
#define DB_PT(PT) \
  cout << #PT << " = " << FORMAT_PT(PT) << endl;


//----------------------------------------------------------------------
// Utility functions
//----------------------------------------------------------------------

/** return object type */
sint4 get_obj_type(const GameObj &gob);

/** get position of an object (as a point) */
Point get_position(const GameObj &gob);

/** calculate distance between two points */
double calc_distance(const Point &a, const Point &b);

/** alternative to Game::distance function */
double calc_distance(const GameObj &a, const GameObj &b);

bool is_unit(const GameObj &gob);

bool has_flag(const GameObj &gob);

Point get_board_position(const GameObj &gob);

bool is_equal(const Point &p1, const Point &p2);

bool is_moving(const GameObj &gob);

sint4 get_hp(const GameObj &obj);

/** convert from a env point to a board point */
Point conv_to_board_point(const Point &pt);

/** convert from board point to env point */
Point conv_to_env_point(const Point &pt);

Point snap_point_to_grid(const Point &pt);

sint4 get_obj_id(const GameObj &gob, Game* game);

#endif
